
//`define ASYNC 0
module bus32_cpu_control (/*AUTOARG*/
  // Outputs
  cpu_bus32_rdata, cpu_bus32_ack_r, cpu_out,
  // Inputs
  bus32_cpu_addr, bus32_cpu_wdata, bus32_cpu_byte_strobes,
  bus32_cpu_write, bus32_cpu_req_r, sel_a, cpu_in, clk, reset
  );


  // Bus32 interface
  input [31:0]      bus32_cpu_addr;
  input [31:0]      bus32_cpu_wdata;
  output [31:0]     cpu_bus32_rdata;
  input [3:0]       bus32_cpu_byte_strobes;
  input             bus32_cpu_write;
            
  input             bus32_cpu_req_r;  // request signal
  output            cpu_bus32_ack_r;

  
  // From "bridge"
  input             sel_a; // asynchronous to the bus32 clock
  

  // I/O
  output [31:0]     cpu_out;
  input  [31:0]     cpu_in;
  

  
  // Clock and reset
  input              clk;
  input              reset;

  
  
  /*AUTOINPUT*/
  /*AUTOOUTPUT*/
  
  /*AUTOREG*/
  // Beginning of automatic regs (for this module's undeclared outputs)
  reg [31:0]            cpu_out;
  // End of automatics
  /*AUTOWIRE*/

  reg                   req_r;
  reg                   req_m;
  reg                   req_d1;

  // inputs are assumed to be asynchronous
  reg [31:0]                  cpu_in_m;
  reg [31:0]                  cpu_in_r;
  
  
  
  //CPU specific inputs and map
  
wire [15:0] addra;
reg [15:0] addrb;
wire [15:0] rf_reg;
reg [3:0] rf_selreg;
wire [15:0] PC_addr;
wire [15:0] pc_r;
assign addra = {1'b0,pc_r[15:1]};
wire [15:0] mem_in;
wire [15:0] mem_addr;
wire mem_cs,mem_we,mem_clk;
wire [15:0] mem_out;
reg wea;
reg [15:0] dinb;
wire [15:0] douta;

wire [3:0]nmenu1_ptr;
wire [3:0]nmenu2_ptr;
wire [3:0]nmenu3_ptr;
wire [15:0]register_out;
wire clkdiv;

//async stuff
reg [3:0] sel_mode;
reg [3:0] ramdsel;
reg [3:0] aludsel;
reg [3:0] rfdsel;
reg [3:0] memdsel;
reg [3:0] clkdsel;
reg [15:0] clkdivsel;
wire cpu_req;
wire cpu_ack;
wire ram_clk;
reg rst;
reg force_reset;

// Force CPU reset
always @* begin
	if(reset== 1) begin
		rst = 1;
	end else begin
		case(force_reset)
			0: begin
				rst = reset;
			end
			1: begin
				rst = 1;
			end
		endcase
	end
end

assign cpu_req = 0;
  

  always @(posedge clk or posedge reset) begin
    if(reset == 1'b1) begin
      /*AUTORESET*/
      // Beginning of autoreset for uninitialized flops
      cpu_in_m <= 32'h0;
      cpu_in_r <= 32'h0;
      cpu_out <= 32'h0;
      req_d1 <= 1'h0;
      req_m <= 1'h0;
      req_r <= 1'h0;
		ramdsel <= 4'b1111;
		clkdsel <= 4'b1111;
		aludsel <= 4'b1111;
		rfdsel  <= 4'b1111;
		memdsel <= 4'b1111;
		clkdivsel <= 16'd10;
		sel_mode <= 4'b1111;
		rf_selreg <= 4'b0000;
		wea <= 0;
		addrb <= 8'b0;
		dinb	<= 8'b0;
		force_reset <= 0;
      // End of automatics
    end
    else begin
      req_m  <= bus32_cpu_req_r;
      req_r  <= req_m;
      req_d1 <= req_r;
      
      cpu_in_m <= cpu_in;
      cpu_in_r <= cpu_in_m;
      
      //DATA PACKET IS DEVIDED AS FOLLOWS
		// |   ID    |    ADDR    |     DATA |   DATA    |
		// 31      24|23        16|15       8|7         0|
		
		
      // Write to register
      // rising edge of request
      // sel_a is stable when req_r is rising
      if (req_r && !req_d1) begin
        // This is probably OK only with DFF with enable (built-in mux)
        // Otherwise could glitches be expected ?
        if(sel_a && bus32_cpu_write) begin
			 wea <= 0;
			 force_reset <= 0;
			 case(bus32_cpu_wdata[31:24]) //3rd byte is module sel
				8'h0: begin
					clkdivsel <= bus32_cpu_wdata[15:0]; //1st byte is data
				end
				8'h1: begin
					case(bus32_cpu_wdata[23:16]) //2nd byte is register
						8'h0: begin
							ramdsel <= bus32_cpu_wdata[3:0];
						end
						8'h1: begin
							aludsel <= bus32_cpu_wdata[3:0];
						end
						8'h2: begin
							rfdsel <= bus32_cpu_wdata[3:0];
						end
						8'h3: begin
							memdsel <= bus32_cpu_wdata[3:0];
						end
						8'h4: begin
							clkdsel <= bus32_cpu_wdata[3:0];
						end
					endcase
				end
				8'h2: begin
					force_reset <= 1;
				end
				8'h3: begin
				 sel_mode <= bus32_cpu_wdata[3:0];
				end
				8'h4: begin
				 wea <= 1;
				 addrb <= bus32_cpu_wdata[24:16];
				 dinb <= bus32_cpu_wdata[15:0];
				end
				8'h5: begin
				 rf_selreg <= bus32_cpu_wdata[3:0];
				end
				endcase
        end else begin
				force_reset <= 0;
				wea <= 0;
			end
      end else begin
			wea <= 0;
			force_reset <= 0;
		end
      
    end
  end
  
  assign cpu_bus32_ack_r = req_r;
  assign cpu_bus32_rdata = {rf_selreg[3:0],12'h0,rf_reg[15:0]};
 
`ifdef ASYNC
simd_useq_async CPU (
  // Outputs
  .pc_r(pc_r[15:0]), .sel_mode(sel_mode[3:0]), .mem_in(mem_in[15:0]), .mem_addr(mem_addr[15:0]), .mem_cs(mem_cs),.mem_we(mem_we),.mem_clk(mem_clk),.cpu_ack(cpu_ack) ,.ram_clk(ram_clk),
  // Inputs
  .instruction(douta[15:0]),.cpu_req(cpu_req), .rst_n(!rst), .mem_out(mem_out[15:0]), .clk(clkdiv), .ramdsel(ramdsel[3:0]),.aludsel(aludsel[3:0]),.rfdsel(rfdsel[3:0]),.memdsel(memdsel[3:0]), .clkdsel(clkdsel[3:0]), .rf_reg(rf_reg[15:0]),.rf_selreg(rf_selreg[3:0])
  );
  
  dualport_progmem PROGMEM (
  .clka(clk), // input clka
  .wea(wea), // input [0 : 0] wea
  .addra(addrb), // input [7 : 0] addra
  .dina(dinb), // input [15 : 0] dina
  .clkb(ram_clk), // input clkb
  .addrb(addra[7:0]), // input [7 : 0] addrb
  .doutb(douta[15:0]) // output [15 : 0] doutb
);

  RAM EXTRAM (
  .clka(mem_clk), // input clka
  .wea(mem_we), // input [0 : 0] wea
  .addra(mem_addr), // input [7 : 0] addra
  .dina(mem_in), // input [15 : 0] dina
  .douta(mem_out) // output [15 : 0] douta
);

`else

simd_useq CPU (
  // Outputs
  .pc_r(pc_r[15:0]), .mem_in(mem_in[15:0]), .mem_addr(mem_addr[15:0]), .mem_cs(mem_cs),.mem_we(mem_we),.mem_clk(mem_clk),
  // Inputs
  .instruction(douta[15:0]), .clk(clkdiv), .rst_n(!rst),.mem_out(mem_out[15:0]), .rf_reg(rf_reg[15:0]),.rf_selreg(rf_selreg[3:0])
  );
  
    dualport_progmem PROGMEM (
  .clka(clk), // input clka
  .wea(wea), // input [0 : 0] wea
  .addra(addrb), // input [7 : 0] addra
  .dina(dinb), // input [15 : 0] dina
  .clkb(clkdiv), // input clkb
  .addrb(addra[7:0]), // input [7 : 0] addrb
  .doutb(douta[15:0]) // output [15 : 0] doutb
);
  
  RAM EXTRAM (
  .clka(clkdiv), // input clka
  .wea(mem_we), // input [0 : 0] wea
  .addra(mem_addr), // input [7 : 0] addra
  .dina(mem_in), // input [15 : 0] dina
  .douta(mem_out) // output [15 : 0] douta
);
`endif

  fredivider CLKDIV (.clk(clk),.rst(rst),.clk_out(clkdiv),.clkdivsel(clkdivsel));

 
endmodule // bus32_simple_cpu
/*  
 Local Variables:
 verilog-library-directories:(
 "."
 )
 End:
 */
