module simd_useq_async(/*AUTOARG*/
  // Outputs
  pc_r, sel_mode, mem_in, mem_addr, mem_cs,mem_we,mem_clk, cpu_ack, ram_clk,
  // Inputs
  instruction, cpu_req, rst_n,mem_out,clk,ramdsel,aludsel,rfdsel,memdsel,clkdselrf_reg,rf_selreg
  );

`include "simd_params.v"
  
  input  [USEQ_INST_MSB:0] instruction; // Instruction word fetched from memory
  output [USEQ_WORD_MSB:0] pc_r; // address for the code memory

  input [3:0] ramdsel;
  input [3:0] aludsel;
  input [3:0] rfdsel;
  input [3:0] memdsel;
  input [3:0] clkdsel;

  input                    rst_n;
  input [3:0] sel_mode;
  input 		  cpu_req;
  input clk;
   input 			[USEQ_WORD_MSB:0]	mem_out;
  output 		cpu_ack;
  output 					ram_clk;
  
     output mem_cs,mem_clk;
  output mem_we;
  output  [USEQ_WORD_MSB:0] mem_addr; 
  output [USEQ_WORD_MSB:0] mem_in;   
   output [USEQ_WORD_MSB:0]			rf_reg;
	input [USEQ_RF_SEL_MSB:0]		rf_selreg;
  /*AUTOINPUT*/
  /*AUTOOUTPUT*/
  
  /*AUTOREG*/
  /*AUTOWIRE*/


  wire [USEQ_WORD_MSB:0]    dec_inst_r;
  wire [USEQ_WORD_MSB:0]		dec_inst_rr;


  // Register file
  reg [3:0]                      rf_sela;
  reg [3:0]                      rf_selb;
  reg [3:0]                      rf_selc;
  reg                            rf_write;
  
  wire [USEQ_WORD_MSB:0]         rf_a;
  wire [USEQ_WORD_MSB:0]         rf_b;
  wire [USEQ_WORD_MSB:0]         rf_c;
  
  // alu
  reg  [USEQ_WORD_MSB:0]          alu_a;
  reg  [USEQ_WORD_MSB:0]          alu_b;
  wire [USEQ_WORD_MSB:0]          alu_c;
                           
  reg [USEQ_WORD_MSB:0]           dest_bus;

  // PC
  reg [USEQ_WORD_MSB:0]           pc_r;
  reg [USEQ_WORD_MSB:0]           npc;
  reg                             write_to_pc;

  // Flow control
  reg                             halt;
  reg                             halt_r;

  
  

  // Special register support
  reg [USEQ_WORD_MSB:0]           sr_read_bus;
  
  // PE array address register
  // Fixme - those must be moved outside uSeq, to the address generator block
  // We keep them here for now for instruction & toolchain tests

  reg [USEQ_WORD_MSB:0]           ar0_r;
  reg [USEQ_WORD_MSB:0]           ar1_r;
  reg [USEQ_WORD_MSB:0]           ar2_r;
  reg [2:0]                       write_ar;
  
  
//@begin[sig_mux_sel]
reg [USEQ_SEL_RFB_MSB:0]  dec_rf_selb_mux_sel;
reg [USEQ_SEL_ALUB_MSB:0]  dec_alu_sourceb_mux_sel;
reg [USEQ_SEL_DEST_MSB:0]  dec_writeback_dest_mux_sel;
reg [USEQ_SEL_PC_MSB:0]  dec_pc_next_mux_sel;
reg [USEQ_SEL_RFC_MSB:0]  dec_rf_selc_mux_sel;
reg [USEQ_SEL_WBSOURCE_MSB:0]  dec_writeback_source_mux_sel;
reg [USEQ_SEL_ALUOP_MSB:0]  dec_alu_op_mux_sel;
reg [USEQ_SEL_ALUA_MSB:0]  dec_alu_sourcea_mux_sel;
reg [USEQ_SEL_RFA_MSB:0]  dec_rf_sela_mux_sel;
reg [USEQ_FLOW_HALT_MSB:0]  dec_flow_halt_mux_sel;
//@end[sig_mux_sel]  

   wire not_ram_ack;
  
   wire mem_cs;
   wire mem_clk;
  reg mem_we;
  reg  [USEQ_WORD_MSB:0] mem_addr; 
  wire [USEQ_WORD_MSB:0] mem_in;   

wire [USEQ_WORD_MSB:0] mem16_out;
reg [USEQ_WORD_MSB:0] mem16_in ;
reg mem16_write;
reg [USEQ_WORD_MSB:0] mem16_addr ;
wire [USEQ_BYTE_MSB:0] mem8_out;
reg [USEQ_BYTE_MSB:0] mem8_in;
reg mem8_write;
reg [USEQ_WORD_MSB:0] mem8_addr;

reg mem_req;
  
		wire ram_clk; 
      wire rf_clk;
 
 
 wire alu_req;
 wire alu_ack;
 
  wire rf_done;
     wire rf_ack;
	  reg ram_req;
  wire ram_ack;
  reg rf_req;
  wire cpu_ack;
    wire mem_clk_pre;
   wire pc_r_clk;
   
     wire ram_clk_async;
  wire rf_clk_async;
  reg mem_clk_async;
  wire dec_clk_async;
  

  
  wire alu_done;
  wire mem_req_pre;
  
assign alu_done = mem_req_pre;
  
  //@begin[inst_decode]
wire [USEQ_INST_FORMAT_OPCODE1_MSB:0] dec_opcode1  = dec_inst_r[USEQ_INST_FORMAT_OPCODE1_OFFSET +: USEQ_INST_FORMAT_OPCODE1_SIZE];

wire [USEQ_INST_FORMAT_RD_MSB:0] dec_rd  = dec_inst_r[USEQ_INST_FORMAT_RD_OFFSET +: USEQ_INST_FORMAT_RD_SIZE];

wire [USEQ_INST_FORMAT_OPCODE3_MSB:0] dec_opcode3  = dec_inst_r[USEQ_INST_FORMAT_OPCODE3_OFFSET +: USEQ_INST_FORMAT_OPCODE3_SIZE];

wire [USEQ_INST_FORMAT_RS2_MSB:0] dec_rs2  = dec_inst_r[USEQ_INST_FORMAT_RS2_OFFSET +: USEQ_INST_FORMAT_RS2_SIZE];

wire [USEQ_INST_FORMAT_UK4_MSB:0] dec_uk4  = dec_inst_r[USEQ_INST_FORMAT_UK4_OFFSET +: USEQ_INST_FORMAT_UK4_SIZE];

wire [USEQ_INST_FORMAT_RS1_MSB:0] dec_rs1  = dec_inst_r[USEQ_INST_FORMAT_RS1_OFFSET +: USEQ_INST_FORMAT_RS1_SIZE];

wire [USEQ_INST_FORMAT_OPCODE4_MSB:0] dec_opcode4  = dec_inst_r[USEQ_INST_FORMAT_OPCODE4_OFFSET +: USEQ_INST_FORMAT_OPCODE4_SIZE];

wire [USEQ_INST_FORMAT_K8_MSB:0] dec_k8  = dec_inst_r[USEQ_INST_FORMAT_K8_OFFSET +: USEQ_INST_FORMAT_K8_SIZE];

wire [USEQ_INST_FORMAT_COND_MSB:0] dec_cond  = dec_inst_r[USEQ_INST_FORMAT_COND_OFFSET +: USEQ_INST_FORMAT_COND_SIZE];

wire [USEQ_INST_FORMAT_K12_MSB:0] dec_k12  = dec_inst_r[USEQ_INST_FORMAT_K12_OFFSET +: USEQ_INST_FORMAT_K12_SIZE];

wire [USEQ_INST_FORMAT_K4_MSB:0] dec_k4  = dec_inst_r[USEQ_INST_FORMAT_K4_OFFSET +: USEQ_INST_FORMAT_K4_SIZE];

  //@end[inst_decode]


  // temporay code for address registers
  always @(rf_done or  rst_n) begin
    if(rst_n == 1'b0) begin
      /*AUTORESET*/
      // Beginning of autoreset for uninitialized flops
      ar0_r <= {(1+(USEQ_WORD_MSB)){1'b0}};
      ar1_r <= {(1+(USEQ_WORD_MSB)){1'b0}};
      ar2_r <= {(1+(USEQ_WORD_MSB)){1'b0}};
      // End of automatics
    end
    else begin
      if(write_ar[2:0] == 3'b001)
        ar0_r <= dest_bus;
      if(write_ar[2:0] == 3'b010)
        ar1_r <= dest_bus;
      if(write_ar[2:0] == 3'b100)
        ar2_r <= dest_bus;
    end
  end

  
	 //might need to store dec_writeback_source_mux_sel to update dest_bus after alu/mem op
	  two_to_one_mux MUXRAMCLK(.in1(ram_clk_async), .in2(clk), .sel(sel_mode[0]), .out(ram_clk));
	 //dec_pc_next_mux_sel_s1
  pulse_generator PGRAM(.out(ram_clk_async),.in(ram_req),.sel(clkdsel));
  pulse_generator PGMEM(.out(mem_clk_pre),.in(mem_req),.sel(clkdsel));
  pulse_generator PGRF(.out(rf_clk_async),.in(rf_req),.sel(clkdsel));
   two_to_one_mux MUXRFCLK (.in1(rf_clk_async), .in2(clk), .sel(sel_mode[1]), .out(rf_clk));
  pulse_generator PGPCR(.out(pc_r_clk),.in(mem_req_pre),.sel(clkdsel));
    pulse_generator PGDEC(.out(dec_clk_async),.in(alu_ack),.sel(clkdsel));
 
   two_to_one_mux MUXMEMCLK (.in1(mem_clk_async), .in2(clk), .sel(sel_mode[2]), .out(mem_clk));
//ram request and delay 
delay_cell #(.DELAY (16)) RAMD (.z(alu_req),.i(ram_req),.sel(ramdsel));
muller_c_element ALUC (.a(alu_req),.b(!rf_done),.reset(rst_n),.c(alu_ack));

 //storage_byte_clk   #(.SIZE (16)) DECS(npc,alu_ack,rf_done,pc_r); 
 
 two_to_one_mux MUXDECCLK(.in1(dec_clk_async), .in2(clk), .sel(sel_mode[3]), .out(dec_clk));
   
 storage_byte_clk   #(.SIZE (16)) DECS(.in(dec_inst_rr),.c(dec_clk),.p(rst_n),.out(dec_inst_r)); 
delay_cell #(.DELAY (16))  ALUD (.z(mem_req_pre),.i(alu_ack),.sel(aludsel));
//MEMREQ_PRE---- rf_req
delay_cell #(.DELAY (16))  RFD (.z(rf_done),.i(rf_req),.sel(rfdsel));
   
   
  always @( rst_n) begin
    if(rst_n == 1'b0) begin
      /*AUTORESET*/
      // Beginning of autoreset for uninitialized flops
       //dec_inst_rr <= {(1+(USEQ_WORD_MSB)){1'b0}};
      // End of automatics
	//alu_req=0;
    end
	end
	
	assign dec_inst_rr = instruction;

  
  


  
  // decoder for register file, selection for port A
  // Constants in the file rf_sela.generated.v (and useq_params.v of course)
  always @* begin
    case(dec_rf_sela_mux_sel)
      USEQ_SEL_RFA_R0: begin
        rf_sela <= 0;        
      end
      USEQ_SEL_RFA_RS1: begin
        rf_sela <= dec_rs1;        
      end
      USEQ_SEL_RFA_RD: begin
        rf_sela <= dec_rd;        
      end
      default:
        rf_sela <= dec_rs1;        
    endcase
  end


  

  // decoder for register file, selection for port B
  // Constants in the file rf_selb.generated.v (and useq_params.v of course)
  
  always @* begin
    case(dec_rf_selb_mux_sel)
	  USEQ_SEL_RFB_RD: begin
		rf_selb <= dec_rd;
	  end
      USEQ_SEL_RFB_R1: begin
        rf_selb <= 'b1;        
      end
      USEQ_SEL_RFB_RS1: begin
        rf_selb <= dec_rs1;        
      end
      USEQ_SEL_RFB_RS2: begin
        rf_selb <= dec_rs2;        
      end
      default:
        rf_selb <= dec_rs2;        
    endcase
  end


  // decoder for register file, selection for port C (writeback)
  // Constants in the file rf_selc.generated.v (and useq_params.v of course)
  always @* begin
    case(dec_rf_selc_mux_sel)
      USEQ_SEL_RFC_R0: begin
        rf_selc <= 0; // Fixme : really needed ?        
      end
      USEQ_SEL_RFC_R1: begin
        rf_selc <= 'b1;        
      end
      USEQ_SEL_RFC_RS2: begin
        rf_selc <= dec_rs2;        
      end
      USEQ_SEL_RFC_RD: begin
        rf_selc <= dec_rd;        
      end
      default:
        rf_selc <= dec_rd;        
    endcase
  end
  

  // decoder for ALU port A
  // see alu_sourcea.generated.v
  always @* begin
    case(dec_alu_sourcea_mux_sel)
      USEQ_SEL_ALUA_RFA: begin
        alu_a <= rf_a;        
      end
      USEQ_SEL_ALUA_NPC: begin
        alu_a <= npc;        
      end
    default:
      alu_a <= rf_a;
    endcase
  end


//  event evt_dbg;
  
  // decoder for ALU port B
  // see alu_sourcea.generated.v
  always @* begin
    case(dec_alu_sourceb_mux_sel)
      USEQ_SEL_ALUB_RFB: begin
        alu_b <= rf_b;        
      end
      USEQ_SEL_ALUB_NOT_RFB: begin
        alu_b <= ~rf_b;        
      end
      USEQ_SEL_ALUB_UK4: begin
        alu_b <= {{(USEQ_WORD_MINUS_K4){1'b0}},dec_uk4};        
      end
      USEQ_SEL_ALUB_K4: begin // Sign extention
        alu_b <= {{(USEQ_WORD_MINUS_K4+1){dec_uk4[3]}},dec_uk4[2:0]};        
      end
      USEQ_SEL_ALUB_K8: begin // Sign extention
        alu_b <= {{(USEQ_WORD_MINUS_K8+1){dec_k8[7]}},dec_k8[6:0]};        
      end
      USEQ_SEL_ALUB_K12: begin // no sign extention               
        alu_b <= {dec_k12,{(USEQ_WORD_MINUS_K12){1'b0}}};        
      end

      USEQ_SEL_ALUB_RFB_OR_UK4: begin
        alu_b <= rf_b | {{(USEQ_WORD_MINUS_K4){1'b0}},dec_uk4};        
      end
      USEQ_SEL_ALUB_NOT_RFB_OR_UK4: begin
        alu_b <= ~(rf_b | {{(USEQ_WORD_MINUS_K4){1'b0}},dec_uk4});        
      end
      USEQ_SEL_ALUB_SR: begin
        alu_b <= sr_read_bus;        
      end
      
      default:
        alu_b <= rf_b;        
    endcase
  end
 

assign mem16_out = mem_out;
assign mem8_out = mem_out;

assign mem_in = alu_c;
assign mem_cs = 0;


  // Write back - source selection
  // see wb_source.generated.v
  always @* begin
    case(dec_writeback_source_mux_sel)
      USEQ_SEL_WBSOURCE_ALU: begin
        dest_bus <= alu_c; 
		mem16_addr <=rf_b;
		mem8_addr <= rf_b;
		mem_addr <= rf_b;
      end
      USEQ_SEL_WBSOURCE_RFB: begin
        dest_bus <= rf_b; 
		mem16_addr <=rf_b;
		mem8_addr <= rf_b;
		mem_addr <= rf_b;
		mem16_in <= alu_c;
		mem8_in <= alu_c;
      end
      USEQ_SEL_WBSOURCE_MEM16: begin
        dest_bus <= mem16_out;
		mem16_addr <=alu_c;
		mem8_addr <=alu_c;
		mem16_in <= alu_c;
		mem_addr <= alu_c;
      end
      USEQ_SEL_WBSOURCE_MEM8: begin
        dest_bus <= mem8_out;
		mem16_addr <=alu_c;
		mem8_addr <=alu_c;
		mem8_in <= alu_c;
		mem_addr <= alu_c;
      end
     default: begin
       dest_bus <= alu_c; 
	   	mem8_addr <=rf_b;
		mem8_in <= alu_c;
		mem16_addr <=rf_b;
		mem16_in <= alu_c;
		mem_addr <= rf_b;
		end
    endcase
  end
  

  assign rf_c = dest_bus;
 // assign mem8_in = alu_c;
 // assign mem16_in = alu_c;

  // Write back - destination selection
  // see wb_dest.generated.v
  always @* begin
    rf_write = 1'b0;
    write_ar = 3'b0;
	mem8_write = 1'b0;
	mem16_write = 1'b0;
	mem_we = 1'b0;
    case(dec_writeback_dest_mux_sel)
      USEQ_SEL_DEST_RFC: begin
        rf_write = 1'b1;        
      end
      USEQ_SEL_DEST_COND_LINK_REG: begin
        rf_write = (dec_cond == USEQ_COND_ALAL);  // we write to the link register only if the condition field
        // is set to USEQ_COND_ALAL (ALways And Link)
      end
      USEQ_SEL_DEST_MEM16: begin
        // Fixme
		mem16_write = 1'b1;
		mem_we = 1'b1;
      end   
      USEQ_SEL_DEST_MEM8: begin
		mem8_write = 1'b1;
        mem_we = 1'b1;
		// Fixme
      end 
      USEQ_SEL_DEST_SR: begin
        case(dec_rd)
         USEQ_SR_AR0 : begin
           write_ar = 3'b001;           
         end
         USEQ_SR_AR1 : begin
           write_ar = 3'b010;           
         end
         USEQ_SR_AR2 : begin
           write_ar = 3'b100;           
         end
          default: begin
            write_ar = 3'b000;           
          end
          
        endcase 
      end 
      default: begin
        rf_write = 1'b0;
        write_ar = 3'b0;
      end
    endcase     
  end


  // Selection of special register for read
  // To be used by mfrs instruction
  
  always @* begin
    case(dec_rs1)
      USEQ_SR_AR0 : begin
        sr_read_bus = ar0_r;        
      end
      USEQ_SR_AR1 : begin
        sr_read_bus = ar1_r;        
      end
      USEQ_SR_AR2 : begin
        sr_read_bus = ar2_r;        
      end
      USEQ_SR_SR : begin
        sr_read_bus = 0; // Fixme        
      end
      default: begin
      end
      
    endcase 
  end

  // Halt instruction
  always @* begin
    case(dec_flow_halt_mux_sel)
      USEQ_FLOW_HALT_YES: begin
        halt = 1'b1;        
      end
      USEQ_FLOW_HALT_NO: begin
        halt = 1'b0;        
      end
      default: begin
        halt = 1'b0;        
      end      
    endcase 
  end
  
  

  

  
  wire [15:0] decode_string;
  
  assign decode_string = {dec_opcode1,dec_opcode3,dec_opcode4,dec_cond};
  reg [39:0]           _codeop_ascii_r;          // Decode of state_r
  initial
    _codeop_ascii_r = "Unknown";
  
  
  always @* begin
    // FIXME : we need the default case
    casez(decode_string)
    //@begin[case_inst]
    USEQ_OPCODES_AND: begin
        _codeop_ascii_r = "USEQ_OPCODES_AND";
        dec_writeback_dest_mux_sel = USEQ_SEL_DEST_RFC;
        dec_writeback_source_mux_sel = USEQ_SEL_WBSOURCE_ALU;
        dec_rf_sela_mux_sel = USEQ_SEL_RFA_RD;
        dec_rf_selb_mux_sel = USEQ_SEL_RFB_RS2;
        dec_rf_selc_mux_sel = USEQ_SEL_RFC_RD;
        dec_alu_sourceb_mux_sel = USEQ_SEL_ALUB_RFB;
        dec_alu_sourcea_mux_sel = USEQ_SEL_ALUA_RFA;
        dec_alu_op_mux_sel = USEQ_SEL_ALUOP_AND;
        dec_flow_halt_mux_sel = USEQ_FLOW_HALT_NO;
        dec_pc_next_mux_sel = USEQ_SEL_PC_INCREMENT;
    end
    USEQ_OPCODES_XNORR1: begin
        _codeop_ascii_r = "USEQ_OPCODES_XNORR1";
        dec_writeback_dest_mux_sel = USEQ_SEL_DEST_RFC;
        dec_writeback_source_mux_sel = USEQ_SEL_WBSOURCE_ALU;
        dec_rf_sela_mux_sel = USEQ_SEL_RFA_RD;
        dec_rf_selb_mux_sel = USEQ_SEL_RFB_R1;
        dec_rf_selc_mux_sel = USEQ_SEL_RFC_RD;
        dec_alu_sourceb_mux_sel = USEQ_SEL_ALUB_RFB_OR_UK4;
        dec_alu_sourcea_mux_sel = USEQ_SEL_ALUA_RFA;
        dec_alu_op_mux_sel = USEQ_SEL_ALUOP_XNOR;
        dec_flow_halt_mux_sel = USEQ_FLOW_HALT_NO;
        dec_pc_next_mux_sel = USEQ_SEL_PC_INCREMENT;
    end
    USEQ_OPCODES_ADD2C: begin
        _codeop_ascii_r = "USEQ_OPCODES_ADD2C";
        dec_writeback_dest_mux_sel = USEQ_SEL_DEST_RFC;
        dec_writeback_source_mux_sel = USEQ_SEL_WBSOURCE_ALU;
        dec_rf_sela_mux_sel = USEQ_SEL_RFA_RD;
        dec_rf_selb_mux_sel = USEQ_SEL_RFB_RS2;
        dec_rf_selc_mux_sel = USEQ_SEL_RFC_RD;
        dec_alu_sourceb_mux_sel = USEQ_SEL_ALUB_RFB;
        dec_alu_sourcea_mux_sel = USEQ_SEL_ALUA_RFA;
        dec_alu_op_mux_sel = USEQ_SEL_ALUOP_ADD;
        dec_flow_halt_mux_sel = USEQ_FLOW_HALT_NO;
        dec_pc_next_mux_sel = USEQ_SEL_PC_INCREMENT;
    end
    USEQ_OPCODES_LDIR1: begin
        _codeop_ascii_r = "USEQ_OPCODES_LDIR1";
        dec_writeback_dest_mux_sel = USEQ_SEL_DEST_RFC;
        dec_writeback_source_mux_sel = USEQ_SEL_WBSOURCE_ALU;
        dec_rf_sela_mux_sel = USEQ_SEL_RFA_R0;
        dec_rf_selb_mux_sel = USEQ_SEL_RFB_RS2;
        dec_rf_selc_mux_sel = USEQ_SEL_RFC_R1;
        dec_alu_sourceb_mux_sel = USEQ_SEL_ALUB_K12;
        dec_alu_sourcea_mux_sel = USEQ_SEL_ALUA_RFA;
        dec_alu_op_mux_sel = USEQ_SEL_ALUOP_ADD;
        dec_flow_halt_mux_sel = USEQ_FLOW_HALT_NO;
        dec_pc_next_mux_sel = USEQ_SEL_PC_INCREMENT;
    end
    USEQ_OPCODES_ORR1: begin
        _codeop_ascii_r = "USEQ_OPCODES_ORR1";
        dec_writeback_dest_mux_sel = USEQ_SEL_DEST_RFC;
        dec_writeback_source_mux_sel = USEQ_SEL_WBSOURCE_ALU;
        dec_rf_sela_mux_sel = USEQ_SEL_RFA_RD;
        dec_rf_selb_mux_sel = USEQ_SEL_RFB_R1;
        dec_rf_selc_mux_sel = USEQ_SEL_RFC_RD;
        dec_alu_sourceb_mux_sel = USEQ_SEL_ALUB_RFB_OR_UK4;
        dec_alu_sourcea_mux_sel = USEQ_SEL_ALUA_RFA;
        dec_alu_op_mux_sel = USEQ_SEL_ALUOP_OR;
        dec_flow_halt_mux_sel = USEQ_FLOW_HALT_NO;
        dec_pc_next_mux_sel = USEQ_SEL_PC_INCREMENT;
    end
    USEQ_OPCODES_HALT: begin
        _codeop_ascii_r = "USEQ_OPCODES_HALT";
        dec_writeback_dest_mux_sel = USEQ_SEL_DEST_RFC;
        dec_writeback_source_mux_sel = USEQ_SEL_WBSOURCE_ALU;
        dec_rf_sela_mux_sel = USEQ_SEL_RFA_R0;
        dec_rf_selb_mux_sel = USEQ_SEL_RFB_RS1;
        dec_rf_selc_mux_sel = USEQ_SEL_RFC_R0;
        dec_alu_sourceb_mux_sel = USEQ_SEL_ALUB_RFB;
        dec_alu_sourcea_mux_sel = USEQ_SEL_ALUA_RFA;
        dec_alu_op_mux_sel = USEQ_SEL_ALUOP_ADD;
        dec_flow_halt_mux_sel = USEQ_FLOW_HALT_YES;
        dec_pc_next_mux_sel = USEQ_SEL_PC_INCREMENT;
    end
    USEQ_OPCODES_XORR1: begin
        _codeop_ascii_r = "USEQ_OPCODES_XORR1";
        dec_writeback_dest_mux_sel = USEQ_SEL_DEST_RFC;
        dec_writeback_source_mux_sel = USEQ_SEL_WBSOURCE_ALU;
        dec_rf_sela_mux_sel = USEQ_SEL_RFA_RD;
        dec_rf_selb_mux_sel = USEQ_SEL_RFB_R1;
        dec_rf_selc_mux_sel = USEQ_SEL_RFC_RD;
        dec_alu_sourceb_mux_sel = USEQ_SEL_ALUB_RFB_OR_UK4;
        dec_alu_sourcea_mux_sel = USEQ_SEL_ALUA_RFA;
        dec_alu_op_mux_sel = USEQ_SEL_ALUOP_XOR;
        dec_flow_halt_mux_sel = USEQ_FLOW_HALT_NO;
        dec_pc_next_mux_sel = USEQ_SEL_PC_INCREMENT;
    end
    USEQ_OPCODES_LB: begin
        _codeop_ascii_r = "USEQ_OPCODES_LB";
        dec_writeback_dest_mux_sel = USEQ_SEL_DEST_RFC;
        dec_writeback_source_mux_sel = USEQ_SEL_WBSOURCE_MEM8;
        dec_rf_sela_mux_sel = USEQ_SEL_RFA_RS1;
        dec_rf_selb_mux_sel = USEQ_SEL_RFB_RD;
        dec_rf_selc_mux_sel = USEQ_SEL_RFC_RD;
        dec_alu_sourceb_mux_sel = USEQ_SEL_ALUB_K4;
        dec_alu_sourcea_mux_sel = USEQ_SEL_ALUA_RFA;
        dec_alu_op_mux_sel = USEQ_SEL_ALUOP_ADD;
        dec_flow_halt_mux_sel = USEQ_FLOW_HALT_NO;
        dec_pc_next_mux_sel = USEQ_SEL_PC_INCREMENT;
    end
    USEQ_OPCODES_SUB2C: begin
        _codeop_ascii_r = "USEQ_OPCODES_SUB2C";
        dec_writeback_dest_mux_sel = USEQ_SEL_DEST_RFC;
        dec_writeback_source_mux_sel = USEQ_SEL_WBSOURCE_ALU;
        dec_rf_sela_mux_sel = USEQ_SEL_RFA_RD;
        dec_rf_selb_mux_sel = USEQ_SEL_RFB_RS2;
        dec_rf_selc_mux_sel = USEQ_SEL_RFC_RD;
        dec_alu_sourceb_mux_sel = USEQ_SEL_ALUB_NOT_RFB;
        dec_alu_sourcea_mux_sel = USEQ_SEL_ALUA_RFA;
        dec_alu_op_mux_sel = USEQ_SEL_ALUOP_ADD;
        dec_flow_halt_mux_sel = USEQ_FLOW_HALT_NO;
        dec_pc_next_mux_sel = USEQ_SEL_PC_INCREMENT;
    end
    USEQ_OPCODES_RET: begin
        _codeop_ascii_r = "USEQ_OPCODES_RET";
        dec_writeback_dest_mux_sel = USEQ_SEL_DEST_PC;
        dec_writeback_source_mux_sel = USEQ_SEL_WBSOURCE_ALU;
        dec_rf_sela_mux_sel = USEQ_SEL_RFA_R0;
        dec_rf_selb_mux_sel = USEQ_SEL_RFB_R14;
        dec_rf_selc_mux_sel = USEQ_SEL_RFC_RD;
        dec_alu_sourceb_mux_sel = USEQ_SEL_ALUB_RFB;
        dec_alu_sourcea_mux_sel = USEQ_SEL_ALUA_RFA;
        dec_alu_op_mux_sel = USEQ_SEL_ALUOP_ADD;
        dec_flow_halt_mux_sel = USEQ_FLOW_HALT_NO;
        dec_pc_next_mux_sel = USEQ_SEL_PC_FROM_ALU;
    end
    USEQ_OPCODES_LW: begin
        _codeop_ascii_r = "USEQ_OPCODES_LW";
        dec_writeback_dest_mux_sel = USEQ_SEL_DEST_RFC;
        dec_writeback_source_mux_sel = USEQ_SEL_WBSOURCE_MEM16;
        dec_rf_sela_mux_sel = USEQ_SEL_RFA_RS1;
        dec_rf_selb_mux_sel = USEQ_SEL_RFB_RD;
        dec_rf_selc_mux_sel = USEQ_SEL_RFC_RD;
        dec_alu_sourceb_mux_sel = USEQ_SEL_ALUB_K4;
        dec_alu_sourcea_mux_sel = USEQ_SEL_ALUA_RFA;
        dec_alu_op_mux_sel = USEQ_SEL_ALUOP_ADD;
        dec_flow_halt_mux_sel = USEQ_FLOW_HALT_NO;
        dec_pc_next_mux_sel = USEQ_SEL_PC_INCREMENT;
    end
    USEQ_OPCODES_SUB3: begin
        _codeop_ascii_r = "USEQ_OPCODES_SUB3";
        dec_writeback_dest_mux_sel = USEQ_SEL_DEST_RFC;
        dec_writeback_source_mux_sel = USEQ_SEL_WBSOURCE_ALU;
        dec_rf_sela_mux_sel = USEQ_SEL_RFA_RS1;
        dec_rf_selb_mux_sel = USEQ_SEL_RFB_RS2;
        dec_rf_selc_mux_sel = USEQ_SEL_RFC_RD;
        dec_alu_sourceb_mux_sel = USEQ_SEL_ALUB_RFB;
        dec_alu_sourcea_mux_sel = USEQ_SEL_ALUA_RFA;
        dec_alu_op_mux_sel = USEQ_SEL_ALUOP_SUB;
        dec_flow_halt_mux_sel = USEQ_FLOW_HALT_NO;
        dec_pc_next_mux_sel = USEQ_SEL_PC_INCREMENT;
    end
    USEQ_OPCODES_XNOR: begin
        _codeop_ascii_r = "USEQ_OPCODES_XNOR";
        dec_writeback_dest_mux_sel = USEQ_SEL_DEST_RFC;
        dec_writeback_source_mux_sel = USEQ_SEL_WBSOURCE_ALU;
        dec_rf_sela_mux_sel = USEQ_SEL_RFA_RD;
        dec_rf_selb_mux_sel = USEQ_SEL_RFB_RS2;
        dec_rf_selc_mux_sel = USEQ_SEL_RFC_RD;
        dec_alu_sourceb_mux_sel = USEQ_SEL_ALUB_RFB;
        dec_alu_sourcea_mux_sel = USEQ_SEL_ALUA_RFA;
        dec_alu_op_mux_sel = USEQ_SEL_ALUOP_XNOR;
        dec_flow_halt_mux_sel = USEQ_FLOW_HALT_NO;
        dec_pc_next_mux_sel = USEQ_SEL_PC_INCREMENT;
    end
    USEQ_OPCODES_BRPC: begin
        _codeop_ascii_r = "USEQ_OPCODES_BRPC";
        dec_writeback_dest_mux_sel = USEQ_SEL_DEST_COND_LINK_REG;
        dec_writeback_source_mux_sel = USEQ_SEL_WBSOURCE_ALU;
        dec_rf_sela_mux_sel = USEQ_SEL_RFA_RD;
        dec_rf_selb_mux_sel = USEQ_SEL_RFB_RS1;
        dec_rf_selc_mux_sel = USEQ_SEL_RFC_RS2;
        dec_alu_sourceb_mux_sel = USEQ_SEL_ALUB_K8;
        dec_alu_sourcea_mux_sel = USEQ_SEL_ALUA_NPC;
        dec_alu_op_mux_sel = USEQ_SEL_ALUOP_ADD;
        dec_flow_halt_mux_sel = USEQ_FLOW_HALT_NO;
        dec_pc_next_mux_sel = USEQ_SEL_PC_COND_LOAD_FROM_ALU;
    end
    USEQ_OPCODES_SUB2: begin
        _codeop_ascii_r = "USEQ_OPCODES_SUB2";
        dec_writeback_dest_mux_sel = USEQ_SEL_DEST_RFC;
        dec_writeback_source_mux_sel = USEQ_SEL_WBSOURCE_ALU;
        dec_rf_sela_mux_sel = USEQ_SEL_RFA_RD;
        dec_rf_selb_mux_sel = USEQ_SEL_RFB_RS1;
        dec_rf_selc_mux_sel = USEQ_SEL_RFC_RD;
        dec_alu_sourceb_mux_sel = USEQ_SEL_ALUB_NOT_RFB_OR_UK4;
        dec_alu_sourcea_mux_sel = USEQ_SEL_ALUA_RFA;
        dec_alu_op_mux_sel = USEQ_SEL_ALUOP_ADD;
        dec_flow_halt_mux_sel = USEQ_FLOW_HALT_NO;
        dec_pc_next_mux_sel = USEQ_SEL_PC_INCREMENT;
    end
    USEQ_OPCODES_ANDR1: begin
        _codeop_ascii_r = "USEQ_OPCODES_ANDR1";
        dec_writeback_dest_mux_sel = USEQ_SEL_DEST_RFC;
        dec_writeback_source_mux_sel = USEQ_SEL_WBSOURCE_ALU;
        dec_rf_sela_mux_sel = USEQ_SEL_RFA_RD;
        dec_rf_selb_mux_sel = USEQ_SEL_RFB_R1;
        dec_rf_selc_mux_sel = USEQ_SEL_RFC_RD;
        dec_alu_sourceb_mux_sel = USEQ_SEL_ALUB_RFB_OR_UK4;
        dec_alu_sourcea_mux_sel = USEQ_SEL_ALUA_RFA;
        dec_alu_op_mux_sel = USEQ_SEL_ALUOP_AND;
        dec_flow_halt_mux_sel = USEQ_FLOW_HALT_NO;
        dec_pc_next_mux_sel = USEQ_SEL_PC_INCREMENT;
    end
    USEQ_OPCODES_MFSR: begin
        _codeop_ascii_r = "USEQ_OPCODES_MFSR";
        dec_writeback_dest_mux_sel = USEQ_SEL_DEST_RFC;
        dec_writeback_source_mux_sel = USEQ_SEL_WBSOURCE_ALU;
        dec_rf_sela_mux_sel = USEQ_SEL_RFA_R0;
        dec_rf_selb_mux_sel = USEQ_SEL_RFB_RS1;
        dec_rf_selc_mux_sel = USEQ_SEL_RFC_RD;
        dec_alu_sourceb_mux_sel = USEQ_SEL_ALUB_SR;
        dec_alu_sourcea_mux_sel = USEQ_SEL_ALUA_RFA;
        dec_alu_op_mux_sel = USEQ_SEL_ALUOP_ADD;
        dec_flow_halt_mux_sel = USEQ_FLOW_HALT_NO;
        dec_pc_next_mux_sel = USEQ_SEL_PC_INCREMENT;
    end
    USEQ_OPCODES_MTSR: begin
        _codeop_ascii_r = "USEQ_OPCODES_MTSR";
        dec_writeback_dest_mux_sel = USEQ_SEL_DEST_SR;
        dec_writeback_source_mux_sel = USEQ_SEL_WBSOURCE_ALU;
        dec_rf_sela_mux_sel = USEQ_SEL_RFA_R0;
        dec_rf_selb_mux_sel = USEQ_SEL_RFB_RS1;
        dec_rf_selc_mux_sel = USEQ_SEL_RFC_RD;
        dec_alu_sourceb_mux_sel = USEQ_SEL_ALUB_RFB;
        dec_alu_sourcea_mux_sel = USEQ_SEL_ALUA_RFA;
        dec_alu_op_mux_sel = USEQ_SEL_ALUOP_ADD;
        dec_flow_halt_mux_sel = USEQ_FLOW_HALT_NO;
        dec_pc_next_mux_sel = USEQ_SEL_PC_INCREMENT;
    end
    USEQ_OPCODES_ORI4: begin
        _codeop_ascii_r = "USEQ_OPCODES_ORI4";
        dec_writeback_dest_mux_sel = USEQ_SEL_DEST_RFC;
        dec_writeback_source_mux_sel = USEQ_SEL_WBSOURCE_ALU;
        dec_rf_sela_mux_sel = USEQ_SEL_RFA_R0;
        dec_rf_selb_mux_sel = USEQ_SEL_RFB_RS1;
        dec_rf_selc_mux_sel = USEQ_SEL_RFC_RD;
        dec_alu_sourceb_mux_sel = USEQ_SEL_ALUB_RFB_OR_UK4;
        dec_alu_sourcea_mux_sel = USEQ_SEL_ALUA_RFA;
        dec_alu_op_mux_sel = USEQ_SEL_ALUOP_OR;
        dec_flow_halt_mux_sel = USEQ_FLOW_HALT_NO;
        dec_pc_next_mux_sel = USEQ_SEL_PC_INCREMENT;
    end
    USEQ_OPCODES_SRAI: begin
        _codeop_ascii_r = "USEQ_OPCODES_SRAI";
        dec_writeback_dest_mux_sel = USEQ_SEL_DEST_RFC;
        dec_writeback_source_mux_sel = USEQ_SEL_WBSOURCE_ALU;
        dec_rf_sela_mux_sel = USEQ_SEL_RFA_RD;
        dec_rf_selb_mux_sel = USEQ_SEL_RFB_R1;
        dec_rf_selc_mux_sel = USEQ_SEL_RFC_RD;
        dec_alu_sourceb_mux_sel = USEQ_SEL_ALUB_UK4;
        dec_alu_sourcea_mux_sel = USEQ_SEL_ALUA_RFA;
        dec_alu_op_mux_sel = USEQ_SEL_ALUOP_SRA;
        dec_flow_halt_mux_sel = USEQ_FLOW_HALT_NO;
        dec_pc_next_mux_sel = USEQ_SEL_PC_INCREMENT;
    end
    USEQ_OPCODES_SEXT: begin
        _codeop_ascii_r = "USEQ_OPCODES_SEXT";
        dec_writeback_dest_mux_sel = USEQ_SEL_DEST_RFC;
        dec_writeback_source_mux_sel = USEQ_SEL_WBSOURCE_ALU;
        dec_rf_sela_mux_sel = USEQ_SEL_RFA_RD;
        dec_rf_selb_mux_sel = USEQ_SEL_RFB_RS2;
        dec_rf_selc_mux_sel = USEQ_SEL_RFC_RD;
        dec_alu_sourceb_mux_sel = USEQ_SEL_ALUB_RFB;
        dec_alu_sourcea_mux_sel = USEQ_SEL_ALUA_RFA;
        dec_alu_op_mux_sel = USEQ_SEL_ALUOP_SEXT;
        dec_flow_halt_mux_sel = USEQ_FLOW_HALT_NO;
        dec_pc_next_mux_sel = USEQ_SEL_PC_INCREMENT;
    end
    USEQ_OPCODES_BR: begin
        _codeop_ascii_r = "USEQ_OPCODES_BR";
        dec_writeback_dest_mux_sel = USEQ_SEL_DEST_COND_LINK_REG;
        dec_writeback_source_mux_sel = USEQ_SEL_WBSOURCE_ALU;
        dec_rf_sela_mux_sel = USEQ_SEL_RFA_R0;
        dec_rf_selb_mux_sel = USEQ_SEL_RFB_RS1;
        dec_rf_selc_mux_sel = USEQ_SEL_RFC_R0;
        dec_alu_sourceb_mux_sel = USEQ_SEL_ALUB_RFB_OR_UK4;
        dec_alu_sourcea_mux_sel = USEQ_SEL_ALUA_RFA;
        dec_alu_op_mux_sel = USEQ_SEL_ALUOP_ADD;
        dec_flow_halt_mux_sel = USEQ_FLOW_HALT_NO;
        dec_pc_next_mux_sel = USEQ_SEL_PC_COND_LOAD_FROM_ALU;
    end
    USEQ_OPCODES_XOR: begin
        _codeop_ascii_r = "USEQ_OPCODES_XOR";
        dec_writeback_dest_mux_sel = USEQ_SEL_DEST_RFC;
        dec_writeback_source_mux_sel = USEQ_SEL_WBSOURCE_ALU;
        dec_rf_sela_mux_sel = USEQ_SEL_RFA_RD;
        dec_rf_selb_mux_sel = USEQ_SEL_RFB_RS2;
        dec_rf_selc_mux_sel = USEQ_SEL_RFC_RD;
        dec_alu_sourceb_mux_sel = USEQ_SEL_ALUB_RFB;
        dec_alu_sourcea_mux_sel = USEQ_SEL_ALUA_RFA;
        dec_alu_op_mux_sel = USEQ_SEL_ALUOP_XOR;
        dec_flow_halt_mux_sel = USEQ_FLOW_HALT_NO;
        dec_pc_next_mux_sel = USEQ_SEL_PC_INCREMENT;
    end
    USEQ_OPCODES_SRA: begin
        _codeop_ascii_r = "USEQ_OPCODES_SRA";
        dec_writeback_dest_mux_sel = USEQ_SEL_DEST_RFC;
        dec_writeback_source_mux_sel = USEQ_SEL_WBSOURCE_ALU;
        dec_rf_sela_mux_sel = USEQ_SEL_RFA_RD;
        dec_rf_selb_mux_sel = USEQ_SEL_RFB_RS2;
        dec_rf_selc_mux_sel = USEQ_SEL_RFC_RD;
        dec_alu_sourceb_mux_sel = USEQ_SEL_ALUB_RFB;
        dec_alu_sourcea_mux_sel = USEQ_SEL_ALUA_RFA;
        dec_alu_op_mux_sel = USEQ_SEL_ALUOP_SRA;
        dec_flow_halt_mux_sel = USEQ_FLOW_HALT_NO;
        dec_pc_next_mux_sel = USEQ_SEL_PC_INCREMENT;
    end
    USEQ_OPCODES_ADD3: begin
        _codeop_ascii_r = "USEQ_OPCODES_ADD3";
        dec_writeback_dest_mux_sel = USEQ_SEL_DEST_RFC;
        dec_writeback_source_mux_sel = USEQ_SEL_WBSOURCE_ALU;
        dec_rf_sela_mux_sel = USEQ_SEL_RFA_RS1;
        dec_rf_selb_mux_sel = USEQ_SEL_RFB_RS2;
        dec_rf_selc_mux_sel = USEQ_SEL_RFC_RD;
        dec_alu_sourceb_mux_sel = USEQ_SEL_ALUB_RFB;
        dec_alu_sourcea_mux_sel = USEQ_SEL_ALUA_RFA;
        dec_alu_op_mux_sel = USEQ_SEL_ALUOP_ADD;
        dec_flow_halt_mux_sel = USEQ_FLOW_HALT_NO;
        dec_pc_next_mux_sel = USEQ_SEL_PC_INCREMENT;
    end
    USEQ_OPCODES_ADD2: begin
        _codeop_ascii_r = "USEQ_OPCODES_ADD2";
        dec_writeback_dest_mux_sel = USEQ_SEL_DEST_RFC;
        dec_writeback_source_mux_sel = USEQ_SEL_WBSOURCE_ALU;
        dec_rf_sela_mux_sel = USEQ_SEL_RFA_RD;
        dec_rf_selb_mux_sel = USEQ_SEL_RFB_RS1;
        dec_rf_selc_mux_sel = USEQ_SEL_RFC_RD;
        dec_alu_sourceb_mux_sel = USEQ_SEL_ALUB_RFB_OR_UK4;
        dec_alu_sourcea_mux_sel = USEQ_SEL_ALUA_RFA;
        dec_alu_op_mux_sel = USEQ_SEL_ALUOP_ADD;
        dec_flow_halt_mux_sel = USEQ_FLOW_HALT_NO;
        dec_pc_next_mux_sel = USEQ_SEL_PC_INCREMENT;
    end
    USEQ_OPCODES_SRLI: begin
        _codeop_ascii_r = "USEQ_OPCODES_SRLI";
        dec_writeback_dest_mux_sel = USEQ_SEL_DEST_RFC;
        dec_writeback_source_mux_sel = USEQ_SEL_WBSOURCE_ALU;
        dec_rf_sela_mux_sel = USEQ_SEL_RFA_RD;
        dec_rf_selb_mux_sel = USEQ_SEL_RFB_R1;
        dec_rf_selc_mux_sel = USEQ_SEL_RFC_RD;
        dec_alu_sourceb_mux_sel = USEQ_SEL_ALUB_UK4;
        dec_alu_sourcea_mux_sel = USEQ_SEL_ALUA_RFA;
        dec_alu_op_mux_sel = USEQ_SEL_ALUOP_SRL;
        dec_flow_halt_mux_sel = USEQ_FLOW_HALT_NO;
        dec_pc_next_mux_sel = USEQ_SEL_PC_INCREMENT;
    end
    USEQ_OPCODES_SW: begin
        _codeop_ascii_r = "USEQ_OPCODES_SW";
        dec_writeback_dest_mux_sel = USEQ_SEL_DEST_MEM16;
        dec_writeback_source_mux_sel = USEQ_SEL_WBSOURCE_RFB;
        dec_rf_sela_mux_sel = USEQ_SEL_RFA_RS1;
        dec_rf_selb_mux_sel = USEQ_SEL_RFB_RD;
        dec_rf_selc_mux_sel = USEQ_SEL_RFC_RD;
        dec_alu_sourceb_mux_sel = USEQ_SEL_ALUB_K4;
        dec_alu_sourcea_mux_sel = USEQ_SEL_ALUA_RFA;
        dec_alu_op_mux_sel = USEQ_SEL_ALUOP_ADD;
        dec_flow_halt_mux_sel = USEQ_FLOW_HALT_NO;
        dec_pc_next_mux_sel = USEQ_SEL_PC_INCREMENT;
    end
    USEQ_OPCODES_SRL: begin
        _codeop_ascii_r = "USEQ_OPCODES_SRL";
        dec_writeback_dest_mux_sel = USEQ_SEL_DEST_RFC;
        dec_writeback_source_mux_sel = USEQ_SEL_WBSOURCE_ALU;
        dec_rf_sela_mux_sel = USEQ_SEL_RFA_RD;
        dec_rf_selb_mux_sel = USEQ_SEL_RFB_RS2;
        dec_rf_selc_mux_sel = USEQ_SEL_RFC_RD;
        dec_alu_sourceb_mux_sel = USEQ_SEL_ALUB_RFB;
        dec_alu_sourcea_mux_sel = USEQ_SEL_ALUA_RFA;
        dec_alu_op_mux_sel = USEQ_SEL_ALUOP_SRL;
        dec_flow_halt_mux_sel = USEQ_FLOW_HALT_NO;
        dec_pc_next_mux_sel = USEQ_SEL_PC_INCREMENT;
    end
    USEQ_OPCODES_SLL: begin
        _codeop_ascii_r = "USEQ_OPCODES_SLL";
        dec_writeback_dest_mux_sel = USEQ_SEL_DEST_RFC;
        dec_writeback_source_mux_sel = USEQ_SEL_WBSOURCE_ALU;
        dec_rf_sela_mux_sel = USEQ_SEL_RFA_RD;
        dec_rf_selb_mux_sel = USEQ_SEL_RFB_RS2;
        dec_rf_selc_mux_sel = USEQ_SEL_RFC_RD;
        dec_alu_sourceb_mux_sel = USEQ_SEL_ALUB_RFB;
        dec_alu_sourcea_mux_sel = USEQ_SEL_ALUA_RFA;
        dec_alu_op_mux_sel = USEQ_SEL_ALUOP_SLL;
        dec_flow_halt_mux_sel = USEQ_FLOW_HALT_NO;
        dec_pc_next_mux_sel = USEQ_SEL_PC_INCREMENT;
    end
    USEQ_OPCODES_SLLI: begin
        _codeop_ascii_r = "USEQ_OPCODES_SLLI";
        dec_writeback_dest_mux_sel = USEQ_SEL_DEST_RFC;
        dec_writeback_source_mux_sel = USEQ_SEL_WBSOURCE_ALU;
        dec_rf_sela_mux_sel = USEQ_SEL_RFA_RD;
        dec_rf_selb_mux_sel = USEQ_SEL_RFB_R1;
        dec_rf_selc_mux_sel = USEQ_SEL_RFC_RD;
        dec_alu_sourceb_mux_sel = USEQ_SEL_ALUB_UK4;
        dec_alu_sourcea_mux_sel = USEQ_SEL_ALUA_RFA;
        dec_alu_op_mux_sel = USEQ_SEL_ALUOP_SLL;
        dec_flow_halt_mux_sel = USEQ_FLOW_HALT_NO;
        dec_pc_next_mux_sel = USEQ_SEL_PC_INCREMENT;
    end
    USEQ_OPCODES_SB: begin
        _codeop_ascii_r = "USEQ_OPCODES_SB";
        dec_writeback_dest_mux_sel = USEQ_SEL_DEST_MEM8;
        dec_writeback_source_mux_sel = USEQ_SEL_WBSOURCE_RFB;
        dec_rf_sela_mux_sel = USEQ_SEL_RFA_RS1;
        dec_rf_selb_mux_sel = USEQ_SEL_RFB_RD;
        dec_rf_selc_mux_sel = USEQ_SEL_RFC_RD;
        dec_alu_sourceb_mux_sel = USEQ_SEL_ALUB_K4;
        dec_alu_sourcea_mux_sel = USEQ_SEL_ALUA_RFA;
        dec_alu_op_mux_sel = USEQ_SEL_ALUOP_ADD;
        dec_flow_halt_mux_sel = USEQ_FLOW_HALT_NO;
        dec_pc_next_mux_sel = USEQ_SEL_PC_INCREMENT;
    end
    USEQ_OPCODES_OR: begin
        _codeop_ascii_r = "USEQ_OPCODES_OR";
        dec_writeback_dest_mux_sel = USEQ_SEL_DEST_RFC;
        dec_writeback_source_mux_sel = USEQ_SEL_WBSOURCE_ALU;
        dec_rf_sela_mux_sel = USEQ_SEL_RFA_RD;
        dec_rf_selb_mux_sel = USEQ_SEL_RFB_RS2;
        dec_rf_selc_mux_sel = USEQ_SEL_RFC_RD;
        dec_alu_sourceb_mux_sel = USEQ_SEL_ALUB_RFB;
        dec_alu_sourcea_mux_sel = USEQ_SEL_ALUA_RFA;
        dec_alu_op_mux_sel = USEQ_SEL_ALUOP_OR;
        dec_flow_halt_mux_sel = USEQ_FLOW_HALT_NO;
        dec_pc_next_mux_sel = USEQ_SEL_PC_INCREMENT;
    end
   //@end[case_inst]
   endcase
  end


  wire cond_branch;
  assign cond_branch = 1'b1; // Fixme
  

 
	
/*
always @* begin
   if(rst_n == 1'b0) begin
		npc <= 0;
		end 
	else if(halt) begin
      npc <= pc_r;      
    end 
	else begin
	
    case(dec_pc_next_mux_sel)
      USEQ_SEL_PC_INCREMENT: begin
         npc <= pc_r + USEQ_PC_INC;
      end
      USEQ_SEL_PC_FROM_ALU: begin
         npc <= dest_bus;
      end
      USEQ_SEL_PC_COND_LOAD_FROM_ALU: begin
        if(cond_branch) begin
         npc <= dest_bus;
        end
        else begin
          npc <= pc_r + USEQ_PC_INC;
        end
	end

      default: begin
        npc <= pc_r + USEQ_PC_INC;
      end
      
    endcase
end	
  end */

  
  always @(posedge pc_r_clk or negedge rst_n) begin
    if(rst_n == 1'b0) begin
      /*AUTORESET*/
      // Beginning of autoreset for uninitialized flops
	  npc <= 0;
      pc_r <= {(1+(USEQ_WORD_MSB)){1'b0}};
      // End of automatics
	  //rf_req<=0 ;
    end
    else begin
	if(halt) begin
		pc_r <= pc_r;
		end else begin
	 case(dec_pc_next_mux_sel)
      USEQ_SEL_PC_INCREMENT: begin
         pc_r <= pc_r + USEQ_PC_INC;
      end
      USEQ_SEL_PC_FROM_ALU: begin
         pc_r <= dest_bus;
      end
      USEQ_SEL_PC_COND_LOAD_FROM_ALU: begin
        if(cond_branch) begin
         pc_r <= dest_bus;
        end
        else begin
          pc_r <= pc_r + USEQ_PC_INC;
        end
	end

      default: begin
        pc_r <= pc_r + USEQ_PC_INC;
      end
      
    endcase
	end
      //pc_r <= npc;
      halt_r <= halt;      
    end
  end // always @ (posedge clk or  rst_n)


  assign carry_in = 0; // FIXME
  
  
  //RAM Read
  
  
  
  
  
  //Micro pipeline is split in RAM -> decode -> register read -> ALU -> Mem8/Mem16/--> Register write
  
  //Event controlled latches
  //Muller C elements


  always @* begin
	if(rst_n == 1'b0) begin
		ram_req <=0;
	//end else if(cpu_req) begin
		
	end	else begin
  		ram_req <= !alu_done;
	end  
  end


  wire mem_req_delayed;
 reg  rf_req_pre;
  delay_cell #(.DELAY (16)) MEMD (.z(mem_req_delayed),.i(mem_req),.sel(memdsel));
  
  
  always @* begin
   if(rst_n == 1'b0) begin
		rf_req = 0;
	end else begin 
		casez(decode_string)
		USEQ_OPCODES_SW: begin
			rf_req = mem_req_delayed;
		end 
		USEQ_OPCODES_SB: begin
			rf_req = mem_req_delayed;
		end 
		USEQ_OPCODES_LB:  begin
			rf_req = mem_req_delayed;
		end 
		USEQ_OPCODES_LW: begin
			rf_req = mem_req_delayed;
		end
		default: begin
		 rf_req = rf_req_pre;
		end
		endcase
		end
 end
  
  
 
   
always @* begin
	if(rst_n == 1'b0) begin
		rf_req_pre = 0;
		mem_req = 0;
		mem_clk_async = 0;
	end
	else begin	
		casez(decode_string)
		USEQ_OPCODES_SW: begin
			mem_req = mem_req_pre;
			mem_clk_async = mem_clk_pre;
		end 
		USEQ_OPCODES_SB: begin
			mem_req = mem_req_pre;
			mem_clk_async = mem_clk_pre;
		end 
		USEQ_OPCODES_LB:  begin
			mem_req = mem_req_pre;
			mem_clk_async = mem_clk_pre;
		end 
		USEQ_OPCODES_LW: begin
			mem_req = mem_req_pre;
			mem_clk_async = mem_clk_pre;
		end
		default: begin
		 mem_req = 0;
		 rf_req_pre = mem_req_pre;
		 mem_clk_async = 0;
		end
		endcase
	end
end
  
  simd_useq_alu U_ALU (
                       // Outputs
                       .c               (alu_c[USEQ_WORD_MSB:0]),
                       // Inputs
                       .a               (alu_a[USEQ_WORD_MSB:0]),
                       .b               (alu_b[USEQ_WORD_MSB:0]),
                       .op              (dec_alu_op_mux_sel[USEQ_SEL_ALUOP_MSB:0]),
                       .carry_in        (carry_in),
                       .carry_out       (carry_out));
  
   /* simd_useq_rf AUTO_TEMPLATE(
    ); */
  simd_useq_rf U_RF (
                     .rf_a              (rf_a[USEQ_WORD_MSB:0]),
                     .rf_b              (rf_b[USEQ_WORD_MSB:0]),
                     /*AUTOINST*/
                     // Inputs
                     .rf_sela           (rf_sela[USEQ_RF_SEL_MSB:0]),
                     .rf_selb           (rf_selb[USEQ_RF_SEL_MSB:0]),
                     .rf_selc           (rf_selc[USEQ_RF_SEL_MSB:0]),
                     .rf_c              (rf_c[USEQ_WORD_MSB:0]),
                     .rf_write          (rf_write),
                     .clk               (rf_clk),
                     .rst_n             (rst_n),
							.rf_reg				  (rf_reg[USEQ_WORD_MSB:0]),
							.rf_selreg			  (rf_selreg[USEQ_RF_SEL_MSB:0]));
  
  

 
endmodule // simd_useq
/*  
 Local Variables:
 verilog-library-directories:(
 "."
 )
 End:
 */
