#!/usr/bin/python
from  array import *
from fpgalink2 import *

import struct

import time

REG_CR=0x00
REG_SR=0x00

REG_TFR_SIZE_LSB = 0x02 # Size register (LSB)  R/W
REG_TFR_SIZE_MSB = 0x03 # Size register (MSB)  R/W

REG_DATAOUT = 0x10; #Dataout (RAM output) register

REG_GP0    = 0x20 # General purpose register   R/W
REG_GP1    = 0x21 # General purpose register   R/W
REG_GP2    = 0x22 # General purpose register   R/W
REG_GP3    = 0x23 # General purpose register   R/W

REG_BUSIF32_IN  = 0x30 # 9 bytes (WO) for a future 32-bits bus interface (4 adrr, 4  data , 1 cmd)
REG_BUSIF32_OUT = 0x31 # 4 bytes (RO) to read the result of a 32bit access read


REG_VERSION_MINOR = 0x7E # Dataout (RAM output) register  RO
REG_VERSION_MAJOR = 0x7F # Dataout (RAM output) register  RO

# Mapping on the bus32
BUS32_GPIO      = 0x00001000
BUS32_TCSPI     = 0x00002000
BUS32_SEVEN_SEG = 0x00003000
BUS32_AD9220    = 0x00004000
BUS32_DAC       = 0x00005000
BUS32_CPU       = 0x00007000

REG_TCSPI_CR    = 0x00
REG_TCSPI_SR    = 0x04
REG_TCSPI_TR    = 0x08
REG_TCSPI_RR    = 0x0C


REG_AD9220_CR    = 0x00
REG_AD9220_SR    = 0x04
REG_AD9220_ACQ   = 0x08

SPI_CMD_SET_SELX0_7  = 1
SPI_CMD_SET_SELX8_15 = 2
SPI_CMD_SET_SELY0_7  = 3
SPI_CMD_SET_SELY8_15 = 4
SPI_CMD_GET_SELX     = 5
SPI_CMD_GET_SELY     = 6

SPI_CMD_DESELECT_XY  = 7

SPI_CMD_SET_LEDS     = 16
SPI_CMD_GET_LEDS     = 17

SPI_CMD_SET_DIGPOT0  = 32
SPI_CMD_SET_DIGPOT1  = 33
SPI_CMD_SET_DIGPOT2  = 34
SPI_CMD_SET_DIGPOT3  = 35
SPI_CMD_SET_DIGPOT4  = 36
SPI_CMD_SET_DIGPOT5  = 37
SPI_CMD_SET_DIGPOT6  = 38
SPI_CMD_SET_DIGPOT7  = 39
SPI_CMD_SET_DIGPOT8  = 40

SPI_CMD_GET_DIGPOT0  = 50
SPI_CMD_GET_DIGPOT1  = 51
SPI_CMD_GET_DIGPOT2  = 52
SPI_CMD_GET_DIGPOT3  = 53
SPI_CMD_GET_DIGPOT4  = 54
SPI_CMD_GET_DIGPOT5  = 55
SPI_CMD_GET_DIGPOT6  = 56
SPI_CMD_GET_DIGPOT7  = 57
SPI_CMD_GET_DIGPOT8  = 58

#127+ : reserved for PVMS system


# for debugging purpose mostly
SPI_CMD_WRITE_DBG0   = 240
SPI_CMD_WRITE_DBG1   = 241
SPI_CMD_READ_DBG0    = 242
SPI_CMD_READ_DBG1    = 243

SPI_CMD_READ_FROM_SLAVE  = 254 # // used by all the read/get commands
SPI_CMD_GET_UC3_VERSION = 255





class nexys2:
    def __init__(self, parent=None):
        self.vp = "1443:0005"
        self.ivp = "1443:0005"
        self.handle = None
        self.isNeroCapable = False
        self.isNeroCapable = False

    def connectFPGA(self):
        self.handle = FLHandle()
        try:
            try:
                self.handle = flOpen(self.vp)
            except FLException, ex:
                print "Loading firmware into %s..." % self.ivp
                flLoadStandardFirmware(self.ivp, self.vp,"D0234");

                print "Awaiting renumeration..."
                if ( not flAwaitDevice(self.vp, 600) ):
                    raise FLException("FPGALink device did not renumerate properly as %s" % self.vp)

                print "Attempting to open connection to FPGALink device %s again..." % self.vp
                self.handle = flOpen(self.vp)
            #else:
            #    raise FLException("Could not open FPGALink device at %s and no initial VID:PID was supplied" % vp)

            print "Connecting USB power to FPGA (Nexys2 only !)..."
            flPortAccess(self.handle, 0x0080, 0x0080);
            fpgalink.flSleep(100)
            print "Checking capability of FPGA communication"
            self.isNeroCapable = flIsNeroCapable(self.handle)
            self.isCommCapable = flIsCommCapable(self.handle)
            if ( self.isNeroCapable ):
                chain = flScanChain(self.handle)
                if ( len(chain) > 0 ):
                    print "The FPGALink device at %s scanned its JTAG chain, yielding:" % self.vp
                    for i in chain:
                        print "  0x%08X" % i
                else:
                    print "The FPGALink device at %s scanned its JTAG chain but did not find any attached devices" % self.vp
            else:
                raise FLException("JTAG chain scan requested but FPGALink device at %s does not support NeroJTAG" % self.vp)


        except FLException, ex:
            print ex
        print("Connection done....")
        #finally:
        #    flClose(self.handle)

    def programFPGA(self,filename):
        print("Programming FPGA with file %s" % filename)
        self.handle = FLHandle()
        try:
            try:
                self.handle = flOpen(self.vp)
            except FLException, ex:
                print "Loading firmware into %s..." % self.ivp
                flLoadStandardFirmware(self.ivp, self.vp,"D0234");

                print "Awaiting renumeration..."
                if ( not flAwaitDevice(self.vp, 600) ):
                    raise FLException("FPGALink device did not renumerate properly as %s" % self.vp)

                print "Attempting to open connection to FPGALink device %s again..." % self.vp
                self.handle = flOpen(self.vp)
            #else:
            #    raise FLException("Could not open FPGALink device at %s and no initial VID:PID was supplied" % vp)

            print "Connecting USB power to FPGA (Nexys2 only !)..."
            flPortAccess(self.handle, 0x0080, 0x0080);
            fpgalink.flSleep(100)
            print "Checking capability of FPGA communication"
            self.isNeroCapable = flIsNeroCapable(self.handle)
            self.isCommCapable = flIsCommCapable(self.handle)
            if ( self.isNeroCapable ):
                chain = flScanChain(self.handle)
                if ( len(chain) > 0 ):
                    print "The FPGALink device at %s scanned its JTAG chain, yielding:" % self.vp
                    for i in chain:
                        print "  0x%08X" % i
                else:
                    print "The FPGALink device at %s scanned its JTAG chain but did not find any attached devices" % self.vp
            else:
                raise FLException("JTAG chain scan requested but FPGALink device at %s does not support NeroJTAG" % self.vp)


            print "Playing \"%s\" into the JTAG chain on FPGALink device %s..." % (filename, self.vp)
            if ( self.isNeroCapable ):
                flPlayXSVF(self.handle, filename)
            else:
                raise FLException("XSVF play requested but device at %s does not support NeroJTAG" % self.vp)


        except FLException, ex:
            print ex
        print("Programming done....")
        #finally:
        #    flClose(self.handle)

    def acquisition(self,channel,number_of_data):
        # setup size register
        flWriteChannel(self.handle, 1000 , REG_TFR_SIZE_LSB,number_of_data % 256)
        flWriteChannel(self.handle, 1000 , REG_TFR_SIZE_MSB,number_of_data / 256)
         # Start
        print "Go (number of data = %d) !" % number_of_data
        flWriteChannel(self.handle, 1000 , REG_CR,0x01)
        while True:
            sr = flReadChannel(self.handle,1000,REG_SR,1)
            if not(sr & 0x01): # bit busy is cleared
                break
        array8bits = flReadChannel(self.handle,1000,REG_DATAOUT,number_of_data)
        adc = self.extract_field(array8bits)
        return adc

    def acquisition_ad9220(self,channel,number_of_data):
        # setup size register
        flWriteChannel(self.handle, 1000 , REG_TFR_SIZE_LSB,number_of_data % 256)
        flWriteChannel(self.handle, 1000 , REG_TFR_SIZE_MSB,number_of_data / 256)
         # Start
        self.bus32_write(BUS32_AD9220+REG_AD9220_ACQ,0x01)
        time.sleep(0.001)
        #flWriteChannel(self.handle, 1000 , REG_CR,0x01)
        # print "Go (number of data = %d) !" % number_of_data
        while True:
            sr = self.bus32_read(BUS32_AD9220+REG_AD9220_SR)
            print "SR=0x%08x" % sr
            if not(sr & 0x01): # bit busy is cleared
                break
        array8bits = flReadChannel(self.handle,1000,REG_DATAOUT,number_of_data)
        adc = self.extract_field(array8bits)
        return adc


#    def extract_field_ad9220(self,array8bits):
#        result_size = len(array8bits) / 4
#        adc0 = array.array('i',[]) # signed int
#        for i in range(result_size):
#            byte0 = array8bits[i*4]
#            byte1 = array8bits[i*4+1]
#            byte2 = array8bits[i*4+2]
#            byte3 = array8bits[i*4+3]
#            raw = byte0 + ((byte1 & 0x0F)<<8)
#
#            signed = (raw ^ 0x00800) - 0x00800
#            print("-I- raw=0x%04x  signed = %d" % (raw,signed))
#            adc0.append(signed)
#        return adc0
#
    def extract_field(self,array8bits):
        result_size = len(array8bits) / 4
        adc0 = array.array('i',[])
        for i in range(result_size):
            byte0 = array8bits[i*4]
            byte1 = array8bits[i*4+1]
            byte2 = array8bits[i*4+2]
            byte3 = array8bits[i*4+3]
            # adc0.append(byte0 + ((byte1 & 0x0F)<<8))
            adc0.append(byte0 + ((byte1 & 0xFF)<<8))
            print adc0
        return adc0


    def bus32_read(self,addr):
        print("-I- Read from  address 0x%08x" % addr)
        # L : unsigned long, < : little endian
        # 9 bytes : addr + data + ctrl (0 for read)
        pstr = struct.pack("<L",addr) + struct.pack("<L",0) + '\x00'
        flWriteChannel(self.handle, 1000 , REG_BUSIF32_IN,bytearray(pstr))
        # Read back the result
        array8bits = flReadChannel(self.handle,1000,REG_BUSIF32_OUT,5)
        print("Result of Read : %r " % array8bits)
        data = ( (array8bits[3]*256 + array8bits[2])*256 + array8bits[1])*256 + array8bits[0]
        print("Read data 0x%08x" % data)
        return data
    def bus32_write(self,addr,data):
        print("-I- Write 0x%x to  address 0x%x" % (data,addr))
        # L : unsigned long, < : little endian
        # 9 bytes : addr + data + ctrl (0 for read)
        pstr = struct.pack("<L",addr) + struct.pack("<L",data) + '\x8F'
        flWriteChannel(self.handle, 1000 , REG_BUSIF32_IN,bytearray(pstr))


if __name__ == "__main__":

    print "Nexys2-1200 control programme"

    nexys2 = nexys2()
    nexys2.programFPGA("TOP.xsvf")
    #nexys2.connectFPGA()
    #nexys2.bus32_write(BUS32_SEVEN_SEG,0x0DCB)
    #test = nexys2.bus32_read(BUS32_SEVEN_SEG)
    #nexys2.bus32_write(BUS32_CPU,0x05000002)
    #nexys2.bus32_read(BUS32_CPU)
    #chan0 = nexys2.acquisition(0,16)

