module bus32_seven_segments (/*AUTOARG*/
  // Outputs
  sseg_out, anode_out, sevenseg_bus32_rdata, sevenseg_bus32_ack_r,
  // Inputs
  bus32_sevenseg_addr, bus32_sevenseg_wdata,
  bus32_sevenseg_byte_strobes, bus32_sevenseg_write,
  bus32_sevenseg_req_r, bus32_sel_a, clk_bus32, reset
  );

  // Seven segments (x4) nexys2 interface
  
  output [7:0] sseg_out;
  output [3:0] anode_out; // a zero must be driven to illuminate the 7-segment (PNP transitor outside)

  // Bus32 interface
  input [31:0]      bus32_sevenseg_addr;
  input [31:0]      bus32_sevenseg_wdata;
  output [31:0]     sevenseg_bus32_rdata;
  input [3:0]       bus32_sevenseg_byte_strobes;
  input             bus32_sevenseg_write;
            
  input             bus32_sevenseg_req_r;  // request signal
  output            sevenseg_bus32_ack_r;

  
  // From "bridge"
  input             bus32_sel_a; // asynchronous to the bus32 clock
  

  
  
  
  // Clock and reset
  input              clk_bus32;
  input              reset;

  
  
  /*AUTOINPUT*/
  /*AUTOOUTPUT*/
  
  /*AUTOREG*/
  /*AUTOWIRE*/

  reg                   req_r;
  reg                   req_m;
  reg                   req_d1;

  reg [31:0]         sevenseg_data_r;
  reg                clk_7seg_d1;
  reg                start_7seg_r;
  
 
  wire [3:0]             anode_out;
  wire [7:0]             sseg_out;

  always @(posedge clk_bus32 or posedge reset) begin
    if(reset == 1'b1) begin
      /*AUTORESET*/
      // Beginning of autoreset for uninitialized flops
      req_d1 <= 1'h0;
      req_m <= 1'h0;
      req_r <= 1'h0;
      sevenseg_data_r <= 32'h0;
      // End of automatics
    end
    else begin
      req_m  <= bus32_sevenseg_req_r;
      req_r  <= req_m;
      req_d1 <= req_r;
      

      
      
      // Write to register
      // rising edge of request
      // bus32_sel_a is stable when req_r is rising
      if (req_r && !req_d1) begin
        // This is probably OK only with DFF with enable (built-in mux)
        // Otherwise could glitches be expected ?
        if(bus32_sel_a && bus32_sevenseg_write) begin
          sevenseg_data_r <= bus32_sevenseg_wdata;          
        end            
      end
      
    end
  end
  
  assign sevenseg_bus32_ack_r = req_r;
  assign sevenseg_bus32_rdata = sevenseg_data_r;


 
  four_by_seven U_FOUR_BY_SEVEN (
                                 // Outputs
                                 .segments              (sseg_out[7:0]),
                                 .anodes_neg            (anode_out[3:0]),
                                 // Inputs
                                 .data                  (sevenseg_data_r[15:0]),
                                 .dots                  (~(sevenseg_data_r[31:28])),
                                 .clock                 (clk_bus32),
                                 .strobe                (start_7seg_r),
                                 .reset                 (reset));
  
  


  /* clock_divider AUTO_TEMPLATE(
    .clk_out               (clk_7seg),
    .clk_in                (clk_bus32),
   .reset_neg              (!reset),
   
    ); */
  clock_divider 
    #(
      .BITS(16)
      )
    U_CLOCK_DIVIDER (
                     .clk_out           (clk_7seg),   
                          /*AUTOINST*/
                     // Inputs
                     .clk_in            (clk_bus32),             // Templated
                     .reset_neg         (!reset));                // Templated



  // Detect rising edge of clk_7seg
  always @(posedge clk_bus32 or posedge reset) begin
    if(reset == 1'b1) begin
      /*AUTORESET*/
      // Beginning of autoreset for uninitialized flops
      clk_7seg_d1 <= 1'h0;
      start_7seg_r <= 1'h0;
      // End of automatics
    end
    else begin
      start_7seg_r <= clk_7seg & !(clk_7seg_d1);
      clk_7seg_d1 <= clk_7seg;
    end
  end // always @ (posedge clock or negedge reset_neg)

  
  
endmodule // bus32_seven_segments

/*  
 Local Variables:
 verilog-library-directories:(
 "."
 )
 End:
 */
