module bus32_simple_gpio (/*AUTOARG*/
  // Outputs
  gpio_bus32_rdata, gpio_bus32_ack_r, gpio_out,
  // Inputs
  bus32_gpio_addr, bus32_gpio_wdata, bus32_gpio_byte_strobes,
  bus32_gpio_write, bus32_gpio_req_r, sel_a, gpio_in, clk, reset
  );


  // Bus32 interface
  input [31:0]      bus32_gpio_addr;
  input [31:0]      bus32_gpio_wdata;
  output [31:0]     gpio_bus32_rdata;
  input [3:0]       bus32_gpio_byte_strobes;
  input             bus32_gpio_write;
            
  input             bus32_gpio_req_r;  // request signal
  output            gpio_bus32_ack_r;

  
  // From "bridge"
  input             sel_a; // asynchronous to the bus32 clock
  

  // I/O
  output [31:0]     gpio_out;
  input  [31:0]     gpio_in;
  

  
  // Clock and reset
  input              clk;
  input              reset;

  
  
  /*AUTOINPUT*/
  /*AUTOOUTPUT*/
  
  /*AUTOREG*/
  // Beginning of automatic regs (for this module's undeclared outputs)
  reg [31:0]            gpio_out;
  // End of automatics
  /*AUTOWIRE*/

  reg                   req_r;
  reg                   req_m;
  reg                   req_d1;

  // inputs are assumed to be asynchronous
  reg [31:0]                  gpio_in_m;
  reg [31:0]                  gpio_in_r;
  
  

  always @(posedge clk or posedge reset) begin
    if(reset == 1'b1) begin
      /*AUTORESET*/
      // Beginning of autoreset for uninitialized flops
      gpio_in_m <= 32'h0;
      gpio_in_r <= 32'h0;
      gpio_out <= 32'h0;
      req_d1 <= 1'h0;
      req_m <= 1'h0;
      req_r <= 1'h0;
      // End of automatics
    end
    else begin
      req_m  <= bus32_gpio_req_r;
      req_r  <= req_m;
      req_d1 <= req_r;
      
      gpio_in_m <= gpio_in;
      gpio_in_r <= gpio_in_m;
      
      
      // Write to register
      // rising edge of request
      // sel_a is stable when req_r is rising
      if (req_r && !req_d1) begin
        // This is probably OK only with DFF with enable (built-in mux)
        // Otherwise could glitches be expected ?
        if(sel_a && bus32_gpio_write) begin
          gpio_out <= bus32_gpio_wdata;          
        end            
      end
      
    end
  end
  
  assign gpio_bus32_ack_r = req_r;
  assign gpio_bus32_rdata = gpio_in_r;
  
endmodule // bus32_simple_gpio
/*  
 Local Variables:
 verilog-library-directories:(
 "."
 )
 End:
 */
