
module regif (/*AUTOARG*/
  // Outputs
  rdata, dpram_next_read, start_r, size_r, regif_bus32_addr,
  regif_bus32_wdata, regif_bus32_byte_strobes, regif_bus32_write,
  regif_bus32_req_r,
  // Inputs
  addr, wdata, write_en, read_en, dpram_dout, busy_r,
  bus32_regif_rdata, bus32_regif_ack_clk_bus32_r, clk_bus32,
  reset_bus32, clk, reset
  );

  parameter REG_ADDR_BITS = 7;

  localparam REG_ADDR_MSB = REG_ADDR_BITS-1;
  
  `include "reg_if_params.v"

  // FPGALINK interface
  input [REG_ADDR_MSB:0] addr;
  input [7:0] wdata;
  output [7:0] rdata;
  
  input       write_en;
  input       read_en;


  // DPRAM IF READ signals
  input [7:0] dpram_dout;
  output      dpram_next_read;
  // output [11:0] dpram_max_address;
  
  // DPRAM READ IF & FPGA side commom signals
  output start_r;
  input  busy_r;
  
  // DPRAM Write if
  output [15:0]size_r;
  

  // 32-bit bus interface

  output [31:0]      regif_bus32_addr;
  output [31:0]      regif_bus32_wdata;
  input [31:0]       bus32_regif_rdata;
  output [3:0]       regif_bus32_byte_strobes;
  output             regif_bus32_write;
            
  output             regif_bus32_req_r;  // request signal
  input              bus32_regif_ack_clk_bus32_r; // acknoledge signal, clocked with the bus32 clock
  input              clk_bus32;
  input              reset_bus32;
  

  
  // Misc
  input clk;
  input reset;



  /*AUTOINPUT*/
  /*AUTOOUTPUT*/
  
  /*AUTOREG*/
  // Beginning of automatic regs (for this module's undeclared outputs)
  reg [7:0]             rdata;
  // End of automatics
  /*AUTOWIRE*/

  
  wire   read_en;
  

  // wire [11:0] dpram_max_address;
  wire dpram_next_read;
  reg  dpram_next_read_pre;
  

  // Control register
  reg start_r;
  wire clear_start_r;
  
  // Status register
  wire  busy_r;
  
  // Size register
  reg [15:0] size_r;
  reg [7:0]  gp0_r;
  reg [7:0]  gp1_r;
  reg [7:0]  gp2_r;
  reg [7:0]  gp3_r;


  // 32-bit bus interface
  reg [2:0] read_counter_r;  // 0 to 4 
  reg [4:0] write_counter_r; // 0 to 9
  wire       reset_read_counter;
  wire       reset_write_counter;
  wire       inc_read_counter;

  reg       reset_write_counter_a;
  reg       inc_read_counter_a;

  wire       inc_write_counter;

  reg [7:0] data_in_r[8:0];  // "in" : from the fpga/usb interface : data to bus32 (addresse, data to be written)
  reg [7:0] data_out;        // "out" : to the fpga/usb interface : data read from bus32
  
  
 

  // FIXME : temporay fix
  assign clear_start_r 		  = busy_r;
  

  
  // assign dpram_max_address[11:0]  = size_r[11:0];
  

  
  // Write, std register
  always @(posedge clk or posedge reset) begin
    if(reset == 1'b1) begin
      // Icarus verilog is not happy if we don't do explicit reset of each byte
      data_in_r[0] <= 0;
      data_in_r[1] <= 0;
      data_in_r[2] <= 0;
      data_in_r[3] <= 0;
      data_in_r[4] <= 0;
      data_in_r[5] <= 0;
      data_in_r[6] <= 0;
      data_in_r[7] <= 0;
      data_in_r[8] <= 0;
      /*AUTORESET*/
      // Beginning of autoreset for uninitialized flops
      gp0_r <= 8'h0;
      gp1_r <= 8'h0;
      gp2_r <= 8'h0;
      gp3_r <= 8'h0;
      size_r <= 16'h0;
      // End of automatics
    end
    else begin
      if(write_en) begin
        case(addr)
          REG_TFR_SIZE_LSB: begin
            size_r[7:0] <= wdata[7:0]; 
          end
          REG_TFR_SIZE_MSB: begin
            size_r[15:8] <= wdata[7:0]; 
          end
          REG_GP0: begin
            gp0_r <= wdata[7:0]; 
          end
          REG_GP1: begin
            gp1_r <= wdata[7:0]; 
          end
          REG_GP2: begin
            gp2_r <= wdata[7:0]; 
          end
          REG_GP3: begin
            gp3_r <= wdata[7:0]; 
          end
          REG_BUSIF32_IN: begin
            data_in_r[write_counter_r] <= wdata[7:0];
          end
          
        endcase // case (addr)
      end
    end
  end

  // Special writes 

  
  always @(posedge clk or posedge reset) begin
    if(reset == 1'b1) begin
      /*AUTORESET*/
      // Beginning of autoreset for uninitialized flops
      start_r <= 1'h0;
      // End of automatics
    end
    else begin
      if (clear_start_r) begin
	start_r <= 1'b0;	
      end
      else 
	if(write_en && (addr == REG_CR)) begin
	  start_r 	     <= wdata[0];	  
	end      
    end
  end
  
  
 

  
  
  



  assign dpram_next_read  = dpram_next_read_pre & read_en;
  //assign dpram_next_read  = dpram_next_read_pre ;
  
  // Read , std registers
  always @* begin
    rdata 	     = 8'h0;
    dpram_next_read_pre  = 1'b0;
    inc_read_counter_a     = 1'b0;
    reset_write_counter_a  = 1'b0;
    
    //if(read_en) begin
      case(addr)
	REG_CR: begin
          rdata[CR_START] = start_r;    
	end
	REG_SR: begin
          rdata = {7'b0,(busy_r | start_r)};    
	end
	REG_TFR_SIZE_LSB: begin
          rdata = size_r[7:0];    
	end
	REG_TFR_SIZE_MSB: begin
          rdata = size_r[15:8];    
	end      
	REG_DATAOUT: begin
          rdata 		 = dpram_dout;
	  dpram_next_read_pre  = 1'b1;	
	end      
	REG_GP0: begin
          rdata = gp0_r;    
	end
	REG_GP1: begin
          rdata = gp1_r;    
	end
	REG_GP2: begin
          rdata = gp2_r;    
	end
	REG_GP3: begin
          rdata = gp3_r;    
	end
        REG_BUSIF32_OUT: begin
          rdata = data_out;
          inc_read_counter_a    = 1'b1 ;
          reset_write_counter_a = 1'b1 ;          
        end
        

        
	REG_VERSION_MINOR: begin
          rdata = VERSION_MINOR;    
	end
	REG_VERSION_MAJOR: begin
          rdata = VERSION_MAJOR;    
	end
	default: begin
	  rdata 	     = 8'h0;
	  dpram_next_read_pre  = 1'b0;
	end
      
      
    endcase // case (addr)
    //end
  end

  // There is a bug in Icarus that prevents moving the qualification with read_en in the case statement above
  assign inc_read_counter    = inc_read_counter_a & read_en;
  assign reset_write_counter =  reset_write_counter_a & read_en;
  
  

  always @(posedge clk or posedge reset) begin
    if(reset == 1'b1) begin
      /*AUTORESET*/
      // Beginning of autoreset for uninitialized flops
      write_counter_r <= 5'h0;
      // End of automatics
    end
    else begin
      if (reset_write_counter || (write_counter_r == 8)) begin
        write_counter_r <= 0;        
      end
      else begin
        if(inc_write_counter) begin
          write_counter_r <= write_counter_r + 1;          
        end
      end      
    end
  end


  always @(posedge clk or posedge reset) begin
    if(reset == 1'b1) begin
      /*AUTORESET*/
      // Beginning of autoreset for uninitialized flops
      read_counter_r <= 3'h0;
      // End of automatics
    end
    else begin
      if (reset_read_counter) begin
        read_counter_r <= 0;        
      end
      else begin
        if(inc_read_counter) begin
          read_counter_r <= read_counter_r + 1;          
        end
      end      
    end
  end
  
  

  // interface to bus32 clock domain
  reg regif_bus32_req_r;
  reg bus32_regif_ack_clk_r;
  reg bus32_regif_ack_clk_m;
  wire bus32_busy;



  // asynchronous write-related signal
  assign inc_write_counter = write_en && (addr == REG_BUSIF32_IN);
  assign reset_read_counter = inc_write_counter;
  assign last_data_a         = write_en && (addr == REG_BUSIF32_IN) && (write_counter_r == 8);



  

  
  always @(posedge clk or posedge reset) begin
    if(reset == 1'b1) begin
      /*AUTORESET*/
      // Beginning of autoreset for uninitialized flops
      bus32_regif_ack_clk_m <= 1'h0;
      bus32_regif_ack_clk_r <= 1'h0;
      regif_bus32_req_r <= 1'h0;
      // End of automatics
    end
    else begin
      // synchronizer
      bus32_regif_ack_clk_m <= bus32_regif_ack_clk_bus32_r;
      bus32_regif_ack_clk_r <= bus32_regif_ack_clk_m;
      
      
      if (last_data_a) begin
        regif_bus32_req_r <= 'b1;        
      end
      else begin
        if (bus32_regif_ack_clk_r) begin
          regif_bus32_req_r <= 'b0;
        end
      end
    end
  end
  

  wire [31:0]      regif_bus32_addr;
  wire [31:0]      regif_bus32_wdata;
  wire [3:0]       regif_bus32_byte_strobes;
  wire             regif_bus32_write;
  
  assign bus32_busy = regif_bus32_req_r | bus32_regif_ack_clk_bus32_r;

  
  assign regif_bus32_addr         = { data_in_r[3],data_in_r[2],data_in_r[1],data_in_r[0]};
  assign regif_bus32_wdata        = { data_in_r[7],data_in_r[6],data_in_r[5],data_in_r[4]};
  assign regif_bus32_byte_strobes = data_in_r[8][3:0];
  assign regif_bus32_write        = data_in_r[8][7];
  
  
  // We register the data for the bus32 domain as soon as the ack signal is going high
  // because there is no garanty that the slave will keep the data forever on the bus
  // (for example, if the bus is a APB 2-cycle bus)
  
  
  reg              ack_clk_bus32_d1;
  reg [31:0]       rdata_clk_r;
  
  always @(posedge clk_bus32 or posedge reset_bus32) begin
    if(reset_bus32 == 1'b1) begin
      /*AUTORESET*/
      // Beginning of autoreset for uninitialized flops
      ack_clk_bus32_d1 <= 1'h0;
      rdata_clk_r <= 32'h0;
      // End of automatics
    end
    else begin
      ack_clk_bus32_d1 <= bus32_regif_ack_clk_bus32_r;
      // rising edge detection
      if(bus32_regif_ack_clk_bus32_r && !ack_clk_bus32_d1) begin
        rdata_clk_r <= bus32_regif_rdata;
      end      
    end
  end
  
  // selector for data_out
  // We use an explicit multiplexer because some stuff is coming from the clk domain and some other stuff from the clk_bus32

  always @* begin
    data_out = 0;
    casex (read_counter_r) 
      3'd0: data_out =  rdata_clk_r[7:0];
      3'd1: data_out =  rdata_clk_r[15:8];
      3'd2: data_out =  rdata_clk_r[23:16];
      3'd3: data_out =  rdata_clk_r[31:24];
      3'd4: begin  // Status Byte
        data_out[0] = bus32_busy;        
      end
    endcase // casex (read_counter_r)
    
  end
  
  


  
 
  
endmodule // regif
/*  
 Local Variables:
 verilog-library-directories:(
 "."
 )
 End:
 */
