module simd_pe_rf (/*AUTOARG*/
  // Outputs
  rf_a, rf_b,
  // Inputs
  rf_sela, rf_selb, rf_selc, rf_c, mem_read, rf_write, rf_write_r0,
  rf_write_r1, clk, rst_n
  );

  parameter NUM_REGS=8;
`include "simd_pe_params.v"
  
  // Selection
  input [PE_RF_SEL_MSB:0] rf_sela;
  input [PE_RF_SEL_MSB:0]  rf_selb;
  input [PE_RF_SEL_MSB:0]   rf_selc;
  // Data ports
  output [PE_WORD_MSB:0] rf_a;
  output [PE_WORD_MSB:0] rf_b;
  input [PE_WORD_MSB:0]  rf_c;
  input [PE_WORD_MSB:0]  mem_read;
  
  input                    rf_write;
  input                    rf_write_r0;
  input                    rf_write_r1;
  input                    clk;
  input                    rst_n;
  
  /*AUTOINPUT*/
  
  /*AUTOOUTPUT*/
  
  /*AUTOREG*/
  
  /*AUTOWIRE*/

  wire [PE_WORD_MSB:0] rf_a;
  wire [PE_WORD_MSB:0] rf_b;

  
  reg [PE_WORD_MSB:0] regfile [0:NUM_REGS-1];

  assign rf_a = regfile[rf_sela];
  assign rf_b = regfile[rf_selb];  

  // R0 & R1 can't be written from the ALU (they are connected to the memory)
  always @(posedge clk) begin
    if((rf_selc != 0) && (rf_selc != 1) && rf_write) begin
      regfile[rf_selc] <= rf_c;
    end
  end

  always @(posedge clk) begin
    if(rf_write_r0 ) begin
      regfile[0] <= mem_read;
    end
    if(rf_write_r1) begin
      regfile[1] <= mem_read;
    end
  end
  
  
 // For debugging
// synthesis translate_off
wire  [31:0] r0 ;
wire  [31:0] r1 ;
wire  [31:0] r2 ;
wire  [31:0] r3 ;
wire  [31:0] r4 ;
wire  [31:0] r5 ;
wire  [31:0] r6 ;
wire  [31:0] r7 ;

assign     r0 = 	regfile[1];
assign     r1 = 	regfile[1];
assign     r2 = 	regfile[2];
assign     r3 = 	regfile[3];
assign     r4 = 	regfile[4];
assign     r5 = 	regfile[5];
assign     r6 = 	regfile[6];
assign     r7 = 	regfile[7];
// synthesis translate_on
  
endmodule // dlx3_regfile
/*  
 Local Variables:
 verilog-library-directories:(
 "."
 )
 End:
 */
