module simd_pe (/*AUTOARG*/);

`include "simd_pe_params.v"

  // Memory interface
  input   [PE_WORD_MSB:0] mem_read;
  output [PE_WORD_MSB:0]  mem_write;

  // Instruction
  input   [PE_INST_MSB:0] pe_inst;

  // Communication
  input   [PE_WORD_MSB:0] from_east;
  input   [PE_WORD_MSB:0] from_west;
  input   [PE_WORD_MSB:0] from_north;
  input   [PE_WORD_MSB:0] from_south;
  output  [PE_WORD_MSB:0] to_others;

  // Broadcast from uSeq
  input   [PE_WORD_MSB:0] from_useq;

  // "global" flag (same as  CM1-2/CM-2)
  // Anded between all PEs then sent to uSeq
  output                  global_flag;
  

  
  /*AUTOINPUT*/
  /*AUTOOUTPUT*/
  
  /*AUTOREG*/
  /*AUTOWIRE*/


  reg [PE_WORD_MSB:0]    dec_inst_r;


  // Fixme - This should be moved outside the PE
  // and shared by a group of PEs (not necessary
  // the whole array) - a cluster of 8 or 16 for ex.
  always @(posedge clk or negedge rst_n) begin
    if(rst_n == 1'b0) begin
      /*AUTORESET*/
    end
    else begin
      dec_inst_r <= pe_inst;
      
    end
  end
  


  
  //@begin[pe_inst_decode]
wire [PE_INST_FORMAT_ALUOP_MSB:0] dec_aluop  = dec_inst_r[PE_INST_FORMAT_ALUOP_OFFSET +: PE_INST_FORMAT_ALUOP_SIZE];

wire [PE_INST_FORMAT_MEM_MSB:0] dec_mem  = dec_inst_r[PE_INST_FORMAT_MEM_OFFSET +: PE_INST_FORMAT_MEM_SIZE];

wire [PE_INST_FORMAT_RD_MSB:0] dec_rd  = dec_inst_r[PE_INST_FORMAT_RD_OFFSET +: PE_INST_FORMAT_RD_SIZE];

wire [PE_INST_FORMAT_AGEN_MSB:0] dec_agen  = dec_inst_r[PE_INST_FORMAT_AGEN_OFFSET +: PE_INST_FORMAT_AGEN_SIZE];

wire [PE_INST_FORMAT_COND_MSB:0] dec_cond  = dec_inst_r[PE_INST_FORMAT_COND_OFFSET +: PE_INST_FORMAT_COND_SIZE];

wire [PE_INST_FORMAT_RS1_MSB:0] dec_rs1  = dec_inst_r[PE_INST_FORMAT_RS1_OFFSET +: PE_INST_FORMAT_RS1_SIZE];

wire [PE_INST_FORMAT_RS2_MSB:0] dec_rs2  = dec_inst_r[PE_INST_FORMAT_RS2_OFFSET +: PE_INST_FORMAT_RS2_SIZE];

  //@end[pe_inst_decode]
  



  
   /* simd_pe_alu AUTO_TEMPLATE(
    ); */
  simd_pe_alu U_ALU (
                     .op                (aluop[PE_SEL_ALUOP_MSB:0]),
                          /*AUTOINST*/
                     // Outputs
                     .c                 (c[PE_WORD_MSB:0]),
                     // Inputs
                     .a                 (a[PE_WORD_MSB:0]),
                     .b                 (b[PE_WORD_MSB:0]),

                     .carry_in          (carry_in),
                     .carry_out         (carry_out));
  
  

  
   /* simd_pe_rf AUTO_TEMPLATE(
    .rf_sela             (dec_rs1),
    .rf_selb             (dec_rs2),
    .rf_selc             (dec_rd),

    ); */
  simd_pe_rf U_RF (
                          /*AUTOINST*/);
  
  



  
endmodule // simd_pe
/*  
 Local Variables:
 verilog-library-directories:(
 "."
 )
 End:
 */
