module simd_pe_alu (/*AUTOARG*/
  // Outputs
  c,
  // Inputs
  a, b, op, carry_in, carry_out
  );

`include "simd_pe_params.v"

  // ALU data port
  input [PE_WORD_MSB:0] a; 
  input [PE_WORD_MSB:0] b;
  output[PE_WORD_MSB:0] c;

  input [PE_SEL_ALUOP_MSB:0] op;
  input                        carry_in;
  input                        carry_out;
  


  /*AUTOINPUT*/
  /*AUTOOUTPUT*/
  
  /*AUTOREG*/
  // Beginning of automatic regs (for this module's undeclared outputs)
  reg [PE_WORD_MSB:0]   c;
  // End of automatics
  /*AUTOWIRE*/


  
  reg          force_carry;
  reg          force_carry_value;

  reg  [PE_WORD_MSB:0] adder_a; 
  reg  [PE_WORD_MSB:0] adder_b;
  wire [PE_WORD_MSB:0]  adder_res;
  wire                   adder_cin;
  

  
  always @* begin
    adder_a = a;
    adder_b = b;
    c = adder_res;    
    force_carry       =1'b0;
    force_carry_value =1'b0;
    case(op)
      PE_SEL_ALUOP_ADD: begin
        force_carry        = 1'b1;
        force_carry_value  = 1'b0;
        c = adder_res;    
      end
      

    endcase // case (op)    
  end
  
  


  assign is_not_equal = (|adder_res);
  assign is_equal = !is_not_equal;
  assign is_less = adder_res[PE_WORD_MSB];
  assign is_greater = !adder_res[PE_WORD_MSB] && is_not_equal;

  assign is_less_or_equal = is_less || is_equal;
  assign is_greater_or_equal = is_greater || is_equal;

  
  assign adder_cin  = force_carry ? force_carry_value : carry_in;
  assign carry_cout = adder_cout;
 
  simd_generic_adder 
    #(.WIDTH(PE_WORD_BITS)) 
  U_ALU_ADDER (             
               // Outputs
               .res              (adder_res[PE_WORD_BITS-1:0]),
               .cout             (adder_cout),
               // Inputs
               .a                (adder_a[PE_WORD_BITS-1:0]),
               .b                (adder_b[PE_WORD_BITS-1:0]),
               .cin              (adder_cin));
  
  

  
endmodule // simd_pe_alu
/*  
 Local Variables:
 verilog-library-directories:(
 "."
 )
 End:
 */
