module simd_useq_rf (/*AUTOARG*/
  // Outputs
  rf_a, rf_b, rf_ack,
  // Inputs
  rf_req, rf_sela, rf_selb, rf_selc, rf_c, rf_write, clk, rst_n
  );

  parameter NUM_REGS=15;
`include "simd_params.v"
  
  // Selection
  input  [USEQ_RF_SEL_MSB:0] rf_sela;
  input [USEQ_RF_SEL_MSB:0]  rf_selb;
  input [USEQ_RF_SEL_MSB:0]   rf_selc;
  // Data ports
  output [USEQ_WORD_MSB:0] rf_a;
  output [USEQ_WORD_MSB:0] rf_b;
  input [USEQ_WORD_MSB:0]  rf_c;
  input                    rf_write;
  input                    clk;
  input                    rst_n; 
  input 					rf_req;
  output					rf_ack;
  
  /*AUTOINPUT*/
  
  /*AUTOOUTPUT*/
  
  /*AUTOREG*/
  // Beginning of automatic regs (for this module's undeclared outputs)
  reg [USEQ_WORD_MSB:0] rf_a;
  reg [USEQ_WORD_MSB:0] rf_b;
  // End of automatics
  
  /*AUTOWIRE*/
 reg rf_ack;



  reg [USEQ_WORD_MSB:0] regfile [1:NUM_REGS-1];

  always @(rf_sela)  begin
    if(rf_sela != 0) begin
      rf_a <= regfile[rf_sela];
    end
    else begin
      rf_a <= 0;
    end
  end


  always @(rf_selb)  begin
    if(rf_selb != 0) begin
      rf_b <= regfile[rf_selb];
    end
    else begin
      rf_b <= 0;
    end
  end
  
  wire rf_req_delayed;
wire rf_req;

  delay_cell D1(rf_req_delayed, rf_req);

  always @(rf_req) begin
    if((rf_selc != 0) && rf_write) begin
      regfile[rf_selc] <= rf_c;
    end
  end

always @(rf_req_delayed or negedge rst_n) begin
	if(rst_n == 1'b0) begin
		rf_ack <= 0;
	end
	else begin 
		rf_ack <= !rf_ack;
	end
end
  
 

  // For debugging

// synthesis translate_off

  
wire  [31:0] r0 ;
wire  [31:0] r1 ;
wire  [31:0] r2 ;
wire  [31:0] r3 ;
wire  [31:0] r4 ;
wire  [31:0] r5 ;
wire  [31:0] r6 ;
wire  [31:0] r7 ;
wire  [31:0] r8 ;
wire  [31:0] r9 ;
wire  [31:0] r10;
wire  [31:0] r11;
wire  [31:0] r12;
wire  [31:0] r13;
wire  [31:0] r14;





assign     r1 = 	regfile[1];
assign     r2 = 	regfile[2];
assign     r3 = 	regfile[3];
assign     r4 = 	regfile[4];
assign     r5 = 	regfile[5];
assign     r6 = 	regfile[6];
assign     r7 = 	regfile[7];
assign     r8 = 	regfile[8];
assign     r9 = 	regfile[9];
assign     r10 = 	regfile[10];
assign     r11 = 	regfile[11];
assign     r12 = 	regfile[12];
assign     r13 = 	regfile[13];
assign     r14 = 	regfile[14];


  
// synthesis translate_on



  
  
endmodule // dlx3_regfile
/*  
 Local Variables:
 verilog-library-directories:(
 "."
 )
 End:
 */
