// Generic 16-bit RAM model, 8-bit write access
`timescale 1ns/1ps


module ram16_8 (/*AUTOARG*/
  // Outputs
  read_data,
  // Inputs
  clock, addr, write_data, cs, we
  );
  
  parameter ADDR_WIDTH = 8;
  parameter RAM_SIZE = 1<< ADDR_WIDTH;



  localparam ADDR_MSB = ADDR_WIDTH-1;

  input clock;
  
  input [ADDR_MSB:0] addr;

  input [15:0]       write_data;


  input              cs;


  input [1:0]        we;



  output [15:0] read_data;

  
  
  
  
  /*AUTOINPUT*/
  
  /*AUTOOUTPUT*/
  
  /*AUTOREG*/
  // Beginning of automatic regs (for this module's undeclared outputs)
  reg [15:0]            read_data;
  // End of automatics
  
  /*AUTOWIRE*/


  reg [7:0]     mem0 [0:RAM_SIZE-1];
  reg [7:0]     mem1 [0:RAM_SIZE-1];
  reg [7:0]     mem2 [0:RAM_SIZE-1];
  reg [7:0]     mem3 [0:RAM_SIZE-1];

  reg [ADDR_MSB:0] addr_latched;
  
  
  always @(posedge clock) begin
    if(cs) begin
      addr_latched <= addr;
    end
  end


  always @(posedge clock) begin
    #3;
    if(cs) 
      read_data <= {mem1[addr_latched[ADDR_MSB:0]],mem0[addr_latched[ADDR_MSB:0]]};
  end
  
  

  // Write

 always @(posedge clock) begin
   if(cs && we[0]) begin
     $display("Memory write 7:0!");
     mem0[addr] <= write_data[7:0];
   end
 end

 always @(posedge clock) begin
   if(cs & we[1]) begin
     $display("Memory write 15:8!");
     mem1[addr] <= write_data[15:8];
   end
 end


  
endmodule // ram16_8
/*  
 Local Variables:
 verilog-library-directories:(
 "."
 )
 End:
 */
