
`timescale 1ns/1ps



module tb_simd (/*AUTOARG*/);
  
`include "simd_params.v"  

  

  
  /*AUTOINPUT*/
  
  /*AUTOOUTPUT*/
  
  /*AUTOREG*/
  
  /*AUTOWIRE*/
  // Beginning of automatic wires (for undeclared instantiated-module outputs)
  wire                  clk;                    // From U_CLOK_GEN of clock_gen.v
  wire [USEQ_WORD_MSB:0] pc_r;                  // From U_SIMD of simd.v
  wire                  rst_n;                  // From U_RESET_GEN of reset_gen.v
  reg			cpu_req;
  // End of automatics


  reg [USEQ_INST_MSB:0] instruction_reg;
  wire cpu_ack;
  wire [USEQ_INST_MSB:0] instruction; 
  
   /* simd AUTO_TEMPLATE(
    .instruction             (instruction_reg[USEQ_INST_MSB:0]),
    ); */
  simd U_SIMD (
                          /*AUTOINST*/
               // Outputs
               .pc_r                    (pc_r[USEQ_WORD_MSB:0]),
		.cpu_ack		(cpu_ack),
		.ram_clk		(ram_clk),
               // Inputs
               .clk                     (clk),
               .instruction             (instruction_reg[USEQ_INST_MSB:0]), // Templated
		.cpu_req		(cpu_req),
               .rst_n                   (rst_n));
  
  
   /* ram AUTO_TEMPLATE(
    .addr                 (pc_r[USEQ_WORD_MSB:1]), // Fixme - truncation...
    .we                   (1'b0),
    .write_data           ({(USEQ_WORD_BITS){1'b0}}),
     .read_data            (instruction[USEQ_INST_MSB:0]),
    .clock                (clk),
    .cs                   (1'b1),
        ); */
  ram #(.WORD_WIDTH(USEQ_WORD_BITS),
        .ADDR_WIDTH(USEQ_WORD_BITS-1)) 
  U_USEQ_CODE_RAM (
                   .read_data           (instruction[USEQ_INST_MSB:0]),
                   /*AUTOINST*/
                   // Inputs
                   .clock               (ram_clk),                   // Templated
                   .addr                (pc_r[USEQ_WORD_MSB:1]), // Templated
                   .write_data          ({(USEQ_WORD_BITS){1'b0}}), // Templated
                   .cs                  (1'b1),                  // Templated
                   .we                  (1'b0));                  // Templated
  


  always @* begin
    instruction_reg <= instruction;    
  end
  initial begin
    instruction_reg <= 0;
	cpu_req <= 0;
	#15;	
	cpu_req <= 1;

    
  end
  
  


  
//   /* ram16_8 AUTO_TEMPLATE(
//  ); */
//ram16_8 U_USEQ_DATA_RAM (
//                        /*AUTOINST*/
//                         // Outputs
//                         .read_data           (read_data[15:0]),
//                         // Inputs
//                         .clock               (clock),
//                         .addr                (addr[ADDR_MSB:0]),
//                         .write_data          (write_data[15:0]),
//                         .cs                  (cs),
//                         .we                  (we[1:0]));
//
//
//

  


   /* clock_gen AUTO_TEMPLATE(
    ); */
  clock_gen U_CLOK_GEN (
                          /*AUTOINST*/
                        // Outputs
                        .clk            (clk));
  
  

   /* reset_gen AUTO_TEMPLATE(
    .reset_n               (rst_n),
    ); */
  reset_gen U_RESET_GEN (
                          /*AUTOINST*/
                         // Outputs
                         .reset_n               (rst_n));         // Templated
  
  


  task load_program_memory;
    reg [1024:0] filename;
    reg [7:0]    memory [2<<USEQ_ADDR_BITS:0]; // 8-bit memory
    integer      i;
    integer      dummy;
    begin

      filename = 0;
      dummy = $value$plusargs("program_memory=%s", filename);
      if(filename ==0) begin
        $display("WARNING! No content specified for program memory");
        end
      else begin
        $readmemh (filename, memory);
        for(i=0; i<(1<<USEQ_ADDR_BITS); i=i+2) begin
         U_USEQ_CODE_RAM.mem0[i/2] = {memory[i+1],memory[i]};
        end
      end
    end
  endtask // load_program_memory

  event evt_finish;
  
  initial
    begin
      load_program_memory;
      $dumpfile("tb_simd.vcd");
      $dumpvars(0,tb_simd);
    end

  initial begin
    #5000;
    $display("-E- Timeout reached !");
    -> evt_finish;
    #10;    
    $finish;
  end

  always @(posedge U_SIMD.U_USEQ.halt_r) begin
    $display("-I- Simulation halted by the CPU - <halt> instruction");
    -> evt_finish; 
    #10;    
    $finish;
  end
    
  
   integer i;
   always @(evt_finish) begin
       $display("r00=0x%04h r01=0x%04h r02=0x%04h r03=0x%04h ",
                0,
                U_SIMD.U_USEQ.U_RF.regfile[1],
                U_SIMD.U_USEQ.U_RF.regfile[2],
                U_SIMD.U_USEQ.U_RF.regfile[3]);
       $display("r04=0x%04h r05=0x%04h r06=0x%04h r07=0x%04h ",
                U_SIMD.U_USEQ.U_RF.regfile[4],
                U_SIMD.U_USEQ.U_RF.regfile[5],
                U_SIMD.U_USEQ.U_RF.regfile[6],
                U_SIMD.U_USEQ.U_RF.regfile[7]);
       $display("r08=0x%04h r09=0x%04h r10=0x%04h r11=0x%04h ",
                U_SIMD.U_USEQ.U_RF.regfile[8],
                U_SIMD.U_USEQ.U_RF.regfile[9],
                U_SIMD.U_USEQ.U_RF.regfile[10],
                U_SIMD.U_USEQ.U_RF.regfile[11]);
       $display("r12=0x%04h r13=0x%04h r14=0x%04h pc=0x%04h ",
                U_SIMD.U_USEQ.U_RF.regfile[12],
                U_SIMD.U_USEQ.U_RF.regfile[13],
                U_SIMD.U_USEQ.U_RF.regfile[14],
                U_SIMD.U_USEQ.pc_r);
     end
  

  
endmodule // tb_pipeline
/*  
 Local Variables:
 verilog-library-directories:(
 "."
 "../../rtl/module/verilog"
 )
 End:
 */
