module pulse_generator(out,in);
	input in;
	output out;
	wire a;
	delay_cell D1(a,in);
	xor(out, a, in);
endmodule

module delay_cell (z,i);
	parameter DELAY = 1;
	input i;
	output z;
	assign #DELAY z = i;
endmodule


module simd_useq (/*AUTOARG*/
  // Outputs
  pc_r, cpu_ack, ram_clk,
  // Inputs
  instruction, cpu_req, clk, rst_n
  );

`include "simd_params.v"
  
  input  [USEQ_INST_MSB:0] instruction; // Instruction word fetched from memory
  output [USEQ_WORD_MSB:0] pc_r; // address for the code memory


  input                    clk;
  input                    rst_n;
  input 		  cpu_req;
  output 		cpu_ack;
  output 					ram_clk;
  
  /*AUTOINPUT*/
  /*AUTOOUTPUT*/
  
  /*AUTOREG*/
  /*AUTOWIRE*/


  reg [USEQ_WORD_MSB:0]    dec_inst_r;


  // Register file
  reg [3:0]                      rf_sela;
  reg [3:0]                      rf_selb;
  reg [3:0]                      rf_selc;
  reg                            rf_write;
  
  wire [USEQ_WORD_MSB:0]         rf_a;
  wire [USEQ_WORD_MSB:0]         rf_b;
  wire [USEQ_WORD_MSB:0]         rf_c;
  
  // alu
  reg  [USEQ_WORD_MSB:0]          alu_a;
  reg  [USEQ_WORD_MSB:0]          alu_b;
  wire [USEQ_WORD_MSB:0]          alu_c;
                           
  reg [USEQ_WORD_MSB:0]           dest_bus;


  // PC
  reg [USEQ_WORD_MSB:0]           pc_r;
  reg [USEQ_WORD_MSB:0]           npc;
  reg                             write_to_pc;

  // Flow control
  reg                             halt;
  reg                             halt_r;

  
  

  // Special register support
  reg [USEQ_WORD_MSB:0]           sr_read_bus;
  
  reg ram_req;
  reg ram_ack;
  reg alu_req;
  reg rf_req;
  reg cpu_ack;
wire ram_clk;

  // PE array address register
  // Fixme - those must be moved outside uSeq, to the address generator block
  // We keep them here for now for instruction & toolchain tests

  reg [USEQ_WORD_MSB:0]           ar0_r;
  reg [USEQ_WORD_MSB:0]           ar1_r;
  reg [USEQ_WORD_MSB:0]           ar2_r;
  reg [2:0]                       write_ar;
  
  
//@begin[sig_mux_sel]
reg [USEQ_SEL_RFB_MSB:0]  dec_rf_selb_mux_sel;
reg [USEQ_SEL_ALUB_MSB:0]  dec_alu_sourceb_mux_sel;
reg [USEQ_SEL_DEST_MSB:0]  dec_writeback_dest_mux_sel;
reg [USEQ_SEL_PC_MSB:0]  dec_pc_next_mux_sel;
reg [USEQ_SEL_RFC_MSB:0]  dec_rf_selc_mux_sel;
reg [USEQ_SEL_WBSOURCE_MSB:0]  dec_writeback_source_mux_sel;
reg [USEQ_SEL_ALUOP_MSB:0]  dec_alu_op_mux_sel;
reg [USEQ_SEL_ALUA_MSB:0]  dec_alu_sourcea_mux_sel;
reg [USEQ_SEL_RFA_MSB:0]  dec_rf_sela_mux_sel;
reg [USEQ_FLOW_HALT_MSB:0]  dec_flow_halt_mux_sel;
//@end[sig_mux_sel]  


 
  
  
  //@begin[inst_decode]
wire [USEQ_INST_FORMAT_OPCODE1_MSB:0] dec_opcode1  = dec_inst_r[USEQ_INST_FORMAT_OPCODE1_OFFSET +: USEQ_INST_FORMAT_OPCODE1_SIZE];

wire [USEQ_INST_FORMAT_RD_MSB:0] dec_rd  = dec_inst_r[USEQ_INST_FORMAT_RD_OFFSET +: USEQ_INST_FORMAT_RD_SIZE];

wire [USEQ_INST_FORMAT_OPCODE3_MSB:0] dec_opcode3  = dec_inst_r[USEQ_INST_FORMAT_OPCODE3_OFFSET +: USEQ_INST_FORMAT_OPCODE3_SIZE];

wire [USEQ_INST_FORMAT_RS2_MSB:0] dec_rs2  = dec_inst_r[USEQ_INST_FORMAT_RS2_OFFSET +: USEQ_INST_FORMAT_RS2_SIZE];

wire [USEQ_INST_FORMAT_UK4_MSB:0] dec_uk4  = dec_inst_r[USEQ_INST_FORMAT_UK4_OFFSET +: USEQ_INST_FORMAT_UK4_SIZE];

wire [USEQ_INST_FORMAT_RS1_MSB:0] dec_rs1  = dec_inst_r[USEQ_INST_FORMAT_RS1_OFFSET +: USEQ_INST_FORMAT_RS1_SIZE];

wire [USEQ_INST_FORMAT_OPCODE4_MSB:0] dec_opcode4  = dec_inst_r[USEQ_INST_FORMAT_OPCODE4_OFFSET +: USEQ_INST_FORMAT_OPCODE4_SIZE];

wire [USEQ_INST_FORMAT_K8_MSB:0] dec_k8  = dec_inst_r[USEQ_INST_FORMAT_K8_OFFSET +: USEQ_INST_FORMAT_K8_SIZE];

wire [USEQ_INST_FORMAT_COND_MSB:0] dec_cond  = dec_inst_r[USEQ_INST_FORMAT_COND_OFFSET +: USEQ_INST_FORMAT_COND_SIZE];

wire [USEQ_INST_FORMAT_K12_MSB:0] dec_k12  = dec_inst_r[USEQ_INST_FORMAT_K12_OFFSET +: USEQ_INST_FORMAT_K12_SIZE];

wire [USEQ_INST_FORMAT_K4_MSB:0] dec_k4  = dec_inst_r[USEQ_INST_FORMAT_K4_OFFSET +: USEQ_INST_FORMAT_K4_SIZE];

  //@end[inst_decode]



  // temporay code for address registers
  always @(rf_ack or negedge rst_n) begin
    if(rst_n == 1'b0) begin
      /*AUTORESET*/
      // Beginning of autoreset for uninitialized flops
      ar0_r <= {(1+(USEQ_WORD_MSB)){1'b0}};
      ar1_r <= {(1+(USEQ_WORD_MSB)){1'b0}};
      ar2_r <= {(1+(USEQ_WORD_MSB)){1'b0}};
      // End of automatics
    end
    else begin
      if(write_ar[2:0] == 3'b001)
        ar0_r <= dest_bus;
      if(write_ar[2:0] == 3'b010)
        ar1_r <= dest_bus;
      if(write_ar[2:0] == 3'b100)
        ar2_r <= dest_bus;
    end
  end
  
  


  
  always @(ram_ack or negedge rst_n) begin
    if(rst_n == 1'b0) begin
      /*AUTORESET*/
      // Beginning of autoreset for uninitialized flops
      dec_inst_r <= {(1+(USEQ_WORD_MSB)){1'b0}};
      // End of automatics
	alu_req=0;
    end
    else begin
      dec_inst_r <= instruction;
	alu_req <= !alu_req;
      
    end
  end
  
  


  
  // decoder for register file, selection for port A
  // Constants in the file rf_sela.generated.v (and useq_params.v of course)
  always @* begin
    case(dec_rf_sela_mux_sel)
      USEQ_SEL_RFA_R0: begin
        rf_sela <= 0;        
      end
      USEQ_SEL_RFA_RS1: begin
        rf_sela <= dec_rs1;        
      end
      USEQ_SEL_RFA_RD: begin
        rf_sela <= dec_rd;        
      end
      default:
        rf_sela <= dec_rs1;        
    endcase
  end


  

  // decoder for register file, selection for port B
  // Constants in the file rf_selb.generated.v (and useq_params.v of course)
  
  always @* begin
    case(dec_rf_selb_mux_sel)
      USEQ_SEL_RFB_R1: begin
        rf_selb <= 'b1;        
      end
      USEQ_SEL_RFB_RS1: begin
        rf_selb <= dec_rs1;        
      end
      USEQ_SEL_RFB_RS2: begin
        rf_selb <= dec_rs2;        
      end
      default:
        rf_selb <= dec_rs2;        
    endcase
  end


  // decoder for register file, selection for port C (writeback)
  // Constants in the file rf_selc.generated.v (and useq_params.v of course)
  always @* begin
    case(dec_rf_selc_mux_sel)
      USEQ_SEL_RFC_R0: begin
        rf_selc <= 0; // Fixme : really needed ?        
      end
      USEQ_SEL_RFC_R1: begin
        rf_selc <= 'b1;        
      end
      USEQ_SEL_RFC_RS2: begin
        rf_selc <= dec_rs2;        
      end
      USEQ_SEL_RFC_RD: begin
        rf_selc <= dec_rd;        
      end
      default:
        rf_selc <= dec_rd;        
    endcase
  end
  

  // decoder for ALU port A
  // see alu_sourcea.generated.v
  always @* begin
    case(dec_alu_sourcea_mux_sel)
      USEQ_SEL_ALUA_RFA: begin
        alu_a <= rf_a;        
      end
      USEQ_SEL_ALUA_NPC: begin
        alu_a <= npc;        
      end
    default:
      alu_a <= rf_a;
    endcase
  end


  event evt_dbg;
  
  // decoder for ALU port B
  // see alu_sourcea.generated.v
  always @* begin
    case(dec_alu_sourceb_mux_sel)
      USEQ_SEL_ALUB_RFB: begin
        alu_b <= rf_b;        
      end
      USEQ_SEL_ALUB_NOT_RFB: begin
        alu_b <= ~rf_b;        
      end
      USEQ_SEL_ALUB_UK4: begin
        alu_b <= {{(USEQ_WORD_MINUS_K4){1'b0}},dec_uk4};        
      end
      USEQ_SEL_ALUB_K4: begin // Sign extention
        alu_b <= {{(USEQ_WORD_MINUS_K4+1){dec_uk4[3]}},dec_uk4[2:0]};        
      end
      USEQ_SEL_ALUB_K8: begin // Sign extention
        alu_b <= {{(USEQ_WORD_MINUS_K8+1){dec_k8[7]}},dec_k8[6:0]};        
      end
      USEQ_SEL_ALUB_K12: begin // no sign extention               
        alu_b <= {dec_k12,{(USEQ_WORD_MINUS_K12){1'b0}}};        
      end

      USEQ_SEL_ALUB_RFB_OR_UK4: begin
        alu_b <= rf_b | {{(USEQ_WORD_MINUS_K4){1'b0}},dec_uk4};        
      end
      USEQ_SEL_ALUB_NOT_RFB_OR_UK4: begin
        alu_b <= ~(rf_b | {{(USEQ_WORD_MINUS_K4){1'b0}},dec_uk4});        
      end
      USEQ_SEL_ALUB_SR: begin
        alu_b <= sr_read_bus;        
      end
      
      default:
        alu_b <= rf_b;        
    endcase
  end

  // Write back - source selection
  // see wb_source.generated.v
  always @* begin
    case(dec_writeback_source_mux_sel)
      USEQ_SEL_WBSOURCE_ALU: begin
        dest_bus <= alu_c; 
      end
      USEQ_SEL_WBSOURCE_RFB: begin
        dest_bus <= rf_b; 
      end
      USEQ_SEL_WBSOURCE_MEM16: begin
        // Fixme
      end
      USEQ_SEL_WBSOURCE_MEM8: begin
        // Fixme
      end
     default:
       dest_bus <= alu_c; 
    endcase
  end
  
  

  assign rf_c = dest_bus;

  // Write back - destination selection
  // see wb_dest.generated.v
  always @* begin
    rf_write = 1'b0;
    write_ar = 3'b0;    
    case(dec_writeback_dest_mux_sel)
      USEQ_SEL_DEST_RFC: begin
        rf_write = 1'b1;        
      end
      USEQ_SEL_DEST_COND_LINK_REG: begin
        rf_write = (dec_cond == USEQ_COND_ALAL);  // we write to the link register only if the condition field
        // is set to USEQ_COND_ALAL (ALways And Link)
      end
      USEQ_SEL_DEST_MEM16: begin
        // Fixme
      end   
      USEQ_SEL_DEST_MEM8: begin
        // Fixme
      end 
      USEQ_SEL_DEST_SR: begin
        case(dec_rd)
         USEQ_SR_AR0 : begin
           write_ar = 3'b001;           
         end
         USEQ_SR_AR1 : begin
           write_ar = 3'b010;           
         end
         USEQ_SR_AR2 : begin
           write_ar = 3'b100;           
         end
          default: begin
            write_ar = 3'b000;           
          end
          
        endcase 
      end 
      default: begin
        rf_write = 1'b0;
        write_ar = 3'b0;
      end
    endcase     
  end


  // Selection of special register for read
  // To be used by mfrs instruction
  
  always @* begin
    case(dec_rs1)
      USEQ_SR_AR0 : begin
        sr_read_bus = ar0_r;        
      end
      USEQ_SR_AR1 : begin
        sr_read_bus = ar1_r;        
      end
      USEQ_SR_AR2 : begin
        sr_read_bus = ar2_r;        
      end
      USEQ_SR_SR : begin
        sr_read_bus = 0; // Fixme        
      end
      default: begin
      end
      
    endcase 
  end

  // Halt instruction
  always @* begin
    case(dec_flow_halt_mux_sel)
      USEQ_FLOW_HALT_YES: begin
        halt = 1'b1;        
      end
      USEQ_FLOW_HALT_NO: begin
        halt = 1'b0;        
      end
      default: begin
        halt = 1'b0;        
      end      
    endcase 
  end
  
  

  

  
  wire [15:0] decode_string;
  
  assign decode_string = {dec_opcode1,dec_opcode3,dec_opcode4,dec_cond};
  reg [39:0]           _codeop_ascii_r;          // Decode of state_r
  initial
    _codeop_ascii_r = "Unknown";
  
  
  always @* begin
    // FIXME : we need the default case
    casez(decode_string)
    //@begin[case_inst]
    USEQ_OPCODES_AND: begin
        _codeop_ascii_r = "USEQ_OPCODES_AND";
        dec_writeback_dest_mux_sel = USEQ_SEL_DEST_RFC;
        dec_writeback_source_mux_sel = USEQ_SEL_WBSOURCE_ALU;
        dec_rf_sela_mux_sel = USEQ_SEL_RFA_RD;
        dec_rf_selb_mux_sel = USEQ_SEL_RFB_RS2;
        dec_rf_selc_mux_sel = USEQ_SEL_RFC_RD;
        dec_alu_sourceb_mux_sel = USEQ_SEL_ALUB_RFB;
        dec_alu_sourcea_mux_sel = USEQ_SEL_ALUA_RFA;
        dec_alu_op_mux_sel = USEQ_SEL_ALUOP_AND;
        dec_flow_halt_mux_sel = USEQ_FLOW_HALT_NO;
        dec_pc_next_mux_sel = USEQ_SEL_PC_INCREMENT;
    end
    USEQ_OPCODES_XNORR1: begin
        _codeop_ascii_r = "USEQ_OPCODES_XNORR1";
        dec_writeback_dest_mux_sel = USEQ_SEL_DEST_RFC;
        dec_writeback_source_mux_sel = USEQ_SEL_WBSOURCE_ALU;
        dec_rf_sela_mux_sel = USEQ_SEL_RFA_RD;
        dec_rf_selb_mux_sel = USEQ_SEL_RFB_R1;
        dec_rf_selc_mux_sel = USEQ_SEL_RFC_RD;
        dec_alu_sourceb_mux_sel = USEQ_SEL_ALUB_RFB_OR_UK4;
        dec_alu_sourcea_mux_sel = USEQ_SEL_ALUA_RFA;
        dec_alu_op_mux_sel = USEQ_SEL_ALUOP_XNOR;
        dec_flow_halt_mux_sel = USEQ_FLOW_HALT_NO;
        dec_pc_next_mux_sel = USEQ_SEL_PC_INCREMENT;
    end
    USEQ_OPCODES_ADD2C: begin
        _codeop_ascii_r = "USEQ_OPCODES_ADD2C";
        dec_writeback_dest_mux_sel = USEQ_SEL_DEST_RFC;
        dec_writeback_source_mux_sel = USEQ_SEL_WBSOURCE_ALU;
        dec_rf_sela_mux_sel = USEQ_SEL_RFA_RD;
        dec_rf_selb_mux_sel = USEQ_SEL_RFB_RS2;
        dec_rf_selc_mux_sel = USEQ_SEL_RFC_RD;
        dec_alu_sourceb_mux_sel = USEQ_SEL_ALUB_RFB;
        dec_alu_sourcea_mux_sel = USEQ_SEL_ALUA_RFA;
        dec_alu_op_mux_sel = USEQ_SEL_ALUOP_ADD;
        dec_flow_halt_mux_sel = USEQ_FLOW_HALT_NO;
        dec_pc_next_mux_sel = USEQ_SEL_PC_INCREMENT;
    end
    USEQ_OPCODES_LDIR1: begin
        _codeop_ascii_r = "USEQ_OPCODES_LDIR1";
        dec_writeback_dest_mux_sel = USEQ_SEL_DEST_RFC;
        dec_writeback_source_mux_sel = USEQ_SEL_WBSOURCE_ALU;
        dec_rf_sela_mux_sel = USEQ_SEL_RFA_R0;
        dec_rf_selb_mux_sel = USEQ_SEL_RFB_RS2;
        dec_rf_selc_mux_sel = USEQ_SEL_RFC_R1;
        dec_alu_sourceb_mux_sel = USEQ_SEL_ALUB_K12;
        dec_alu_sourcea_mux_sel = USEQ_SEL_ALUA_RFA;
        dec_alu_op_mux_sel = USEQ_SEL_ALUOP_ADD;
        dec_flow_halt_mux_sel = USEQ_FLOW_HALT_NO;
        dec_pc_next_mux_sel = USEQ_SEL_PC_INCREMENT;
    end
    USEQ_OPCODES_ORR1: begin
        _codeop_ascii_r = "USEQ_OPCODES_ORR1";
        dec_writeback_dest_mux_sel = USEQ_SEL_DEST_RFC;
        dec_writeback_source_mux_sel = USEQ_SEL_WBSOURCE_ALU;
        dec_rf_sela_mux_sel = USEQ_SEL_RFA_RD;
        dec_rf_selb_mux_sel = USEQ_SEL_RFB_R1;
        dec_rf_selc_mux_sel = USEQ_SEL_RFC_RD;
        dec_alu_sourceb_mux_sel = USEQ_SEL_ALUB_RFB_OR_UK4;
        dec_alu_sourcea_mux_sel = USEQ_SEL_ALUA_RFA;
        dec_alu_op_mux_sel = USEQ_SEL_ALUOP_OR;
        dec_flow_halt_mux_sel = USEQ_FLOW_HALT_NO;
        dec_pc_next_mux_sel = USEQ_SEL_PC_INCREMENT;
    end
    USEQ_OPCODES_HALT: begin
        _codeop_ascii_r = "USEQ_OPCODES_HALT";
        dec_writeback_dest_mux_sel = USEQ_SEL_DEST_RFC;
        dec_writeback_source_mux_sel = USEQ_SEL_WBSOURCE_ALU;
        dec_rf_sela_mux_sel = USEQ_SEL_RFA_R0;
        dec_rf_selb_mux_sel = USEQ_SEL_RFB_RS1;
        dec_rf_selc_mux_sel = USEQ_SEL_RFC_R0;
        dec_alu_sourceb_mux_sel = USEQ_SEL_ALUB_RFB;
        dec_alu_sourcea_mux_sel = USEQ_SEL_ALUA_RFA;
        dec_alu_op_mux_sel = USEQ_SEL_ALUOP_ADD;
        dec_flow_halt_mux_sel = USEQ_FLOW_HALT_YES;
        dec_pc_next_mux_sel = USEQ_SEL_PC_INCREMENT;
    end
    USEQ_OPCODES_XORR1: begin
        _codeop_ascii_r = "USEQ_OPCODES_XORR1";
        dec_writeback_dest_mux_sel = USEQ_SEL_DEST_RFC;
        dec_writeback_source_mux_sel = USEQ_SEL_WBSOURCE_ALU;
        dec_rf_sela_mux_sel = USEQ_SEL_RFA_RD;
        dec_rf_selb_mux_sel = USEQ_SEL_RFB_R1;
        dec_rf_selc_mux_sel = USEQ_SEL_RFC_RD;
        dec_alu_sourceb_mux_sel = USEQ_SEL_ALUB_RFB_OR_UK4;
        dec_alu_sourcea_mux_sel = USEQ_SEL_ALUA_RFA;
        dec_alu_op_mux_sel = USEQ_SEL_ALUOP_XOR;
        dec_flow_halt_mux_sel = USEQ_FLOW_HALT_NO;
        dec_pc_next_mux_sel = USEQ_SEL_PC_INCREMENT;
    end
    USEQ_OPCODES_LB: begin
        _codeop_ascii_r = "USEQ_OPCODES_LB";
        dec_writeback_dest_mux_sel = USEQ_SEL_DEST_RFC;
        dec_writeback_source_mux_sel = USEQ_SEL_WBSOURCE_MEM8;
        dec_rf_sela_mux_sel = USEQ_SEL_RFA_RS1;
        dec_rf_selb_mux_sel = USEQ_SEL_RFB_RS1;
        dec_rf_selc_mux_sel = USEQ_SEL_RFC_RD;
        dec_alu_sourceb_mux_sel = USEQ_SEL_ALUB_K4;
        dec_alu_sourcea_mux_sel = USEQ_SEL_ALUA_RFA;
        dec_alu_op_mux_sel = USEQ_SEL_ALUOP_ADD;
        dec_flow_halt_mux_sel = USEQ_FLOW_HALT_NO;
        dec_pc_next_mux_sel = USEQ_SEL_PC_INCREMENT;
    end
    USEQ_OPCODES_SUB2C: begin
        _codeop_ascii_r = "USEQ_OPCODES_SUB2C";
        dec_writeback_dest_mux_sel = USEQ_SEL_DEST_RFC;
        dec_writeback_source_mux_sel = USEQ_SEL_WBSOURCE_ALU;
        dec_rf_sela_mux_sel = USEQ_SEL_RFA_RD;
        dec_rf_selb_mux_sel = USEQ_SEL_RFB_RS2;
        dec_rf_selc_mux_sel = USEQ_SEL_RFC_RD;
        dec_alu_sourceb_mux_sel = USEQ_SEL_ALUB_NOT_RFB;
        dec_alu_sourcea_mux_sel = USEQ_SEL_ALUA_RFA;
        dec_alu_op_mux_sel = USEQ_SEL_ALUOP_ADD;
        dec_flow_halt_mux_sel = USEQ_FLOW_HALT_NO;
        dec_pc_next_mux_sel = USEQ_SEL_PC_INCREMENT;
    end
    USEQ_OPCODES_RET: begin
        _codeop_ascii_r = "USEQ_OPCODES_RET";
        dec_writeback_dest_mux_sel = USEQ_SEL_DEST_PC;
        dec_writeback_source_mux_sel = USEQ_SEL_WBSOURCE_ALU;
        dec_rf_sela_mux_sel = USEQ_SEL_RFA_R0;
        dec_rf_selb_mux_sel = USEQ_SEL_RFB_R14;
        dec_rf_selc_mux_sel = USEQ_SEL_RFC_RD;
        dec_alu_sourceb_mux_sel = USEQ_SEL_ALUB_RFB;
        dec_alu_sourcea_mux_sel = USEQ_SEL_ALUA_RFA;
        dec_alu_op_mux_sel = USEQ_SEL_ALUOP_ADD;
        dec_flow_halt_mux_sel = USEQ_FLOW_HALT_NO;
        dec_pc_next_mux_sel = USEQ_SEL_PC_FROM_ALU;
    end
    USEQ_OPCODES_LW: begin
        _codeop_ascii_r = "USEQ_OPCODES_LW";
        dec_writeback_dest_mux_sel = USEQ_SEL_DEST_RFC;
        dec_writeback_source_mux_sel = USEQ_SEL_WBSOURCE_MEM16;
        dec_rf_sela_mux_sel = USEQ_SEL_RFA_RS1;
        dec_rf_selb_mux_sel = USEQ_SEL_RFB_RS1;
        dec_rf_selc_mux_sel = USEQ_SEL_RFC_RD;
        dec_alu_sourceb_mux_sel = USEQ_SEL_ALUB_K4;
        dec_alu_sourcea_mux_sel = USEQ_SEL_ALUA_RFA;
        dec_alu_op_mux_sel = USEQ_SEL_ALUOP_ADD;
        dec_flow_halt_mux_sel = USEQ_FLOW_HALT_NO;
        dec_pc_next_mux_sel = USEQ_SEL_PC_INCREMENT;
    end
    USEQ_OPCODES_SUB3: begin
        _codeop_ascii_r = "USEQ_OPCODES_SUB3";
        dec_writeback_dest_mux_sel = USEQ_SEL_DEST_RFC;
        dec_writeback_source_mux_sel = USEQ_SEL_WBSOURCE_ALU;
        dec_rf_sela_mux_sel = USEQ_SEL_RFA_RS1;
        dec_rf_selb_mux_sel = USEQ_SEL_RFB_RS2;
        dec_rf_selc_mux_sel = USEQ_SEL_RFC_RD;
        dec_alu_sourceb_mux_sel = USEQ_SEL_ALUB_RFB;
        dec_alu_sourcea_mux_sel = USEQ_SEL_ALUA_RFA;
        dec_alu_op_mux_sel = USEQ_SEL_ALUOP_SUB;
        dec_flow_halt_mux_sel = USEQ_FLOW_HALT_NO;
        dec_pc_next_mux_sel = USEQ_SEL_PC_INCREMENT;
    end
    USEQ_OPCODES_XNOR: begin
        _codeop_ascii_r = "USEQ_OPCODES_XNOR";
        dec_writeback_dest_mux_sel = USEQ_SEL_DEST_RFC;
        dec_writeback_source_mux_sel = USEQ_SEL_WBSOURCE_ALU;
        dec_rf_sela_mux_sel = USEQ_SEL_RFA_RD;
        dec_rf_selb_mux_sel = USEQ_SEL_RFB_RS2;
        dec_rf_selc_mux_sel = USEQ_SEL_RFC_RD;
        dec_alu_sourceb_mux_sel = USEQ_SEL_ALUB_RFB;
        dec_alu_sourcea_mux_sel = USEQ_SEL_ALUA_RFA;
        dec_alu_op_mux_sel = USEQ_SEL_ALUOP_XNOR;
        dec_flow_halt_mux_sel = USEQ_FLOW_HALT_NO;
        dec_pc_next_mux_sel = USEQ_SEL_PC_INCREMENT;
    end
    USEQ_OPCODES_BRPC: begin
        _codeop_ascii_r = "USEQ_OPCODES_BRPC";
        dec_writeback_dest_mux_sel = USEQ_SEL_DEST_COND_LINK_REG;
        dec_writeback_source_mux_sel = USEQ_SEL_WBSOURCE_ALU;
        dec_rf_sela_mux_sel = USEQ_SEL_RFA_RD;
        dec_rf_selb_mux_sel = USEQ_SEL_RFB_RS1;
        dec_rf_selc_mux_sel = USEQ_SEL_RFC_RS2;
        dec_alu_sourceb_mux_sel = USEQ_SEL_ALUB_K8;
        dec_alu_sourcea_mux_sel = USEQ_SEL_ALUA_NPC;
        dec_alu_op_mux_sel = USEQ_SEL_ALUOP_ADD;
        dec_flow_halt_mux_sel = USEQ_FLOW_HALT_NO;
        dec_pc_next_mux_sel = USEQ_SEL_PC_COND_LOAD_FROM_ALU;
    end
    USEQ_OPCODES_SUB2: begin
        _codeop_ascii_r = "USEQ_OPCODES_SUB2";
        dec_writeback_dest_mux_sel = USEQ_SEL_DEST_RFC;
        dec_writeback_source_mux_sel = USEQ_SEL_WBSOURCE_ALU;
        dec_rf_sela_mux_sel = USEQ_SEL_RFA_RD;
        dec_rf_selb_mux_sel = USEQ_SEL_RFB_RS1;
        dec_rf_selc_mux_sel = USEQ_SEL_RFC_RD;
        dec_alu_sourceb_mux_sel = USEQ_SEL_ALUB_NOT_RFB_OR_UK4;
        dec_alu_sourcea_mux_sel = USEQ_SEL_ALUA_RFA;
        dec_alu_op_mux_sel = USEQ_SEL_ALUOP_ADD;
        dec_flow_halt_mux_sel = USEQ_FLOW_HALT_NO;
        dec_pc_next_mux_sel = USEQ_SEL_PC_INCREMENT;
    end
    USEQ_OPCODES_ANDR1: begin
        _codeop_ascii_r = "USEQ_OPCODES_ANDR1";
        dec_writeback_dest_mux_sel = USEQ_SEL_DEST_RFC;
        dec_writeback_source_mux_sel = USEQ_SEL_WBSOURCE_ALU;
        dec_rf_sela_mux_sel = USEQ_SEL_RFA_RD;
        dec_rf_selb_mux_sel = USEQ_SEL_RFB_R1;
        dec_rf_selc_mux_sel = USEQ_SEL_RFC_RD;
        dec_alu_sourceb_mux_sel = USEQ_SEL_ALUB_RFB_OR_UK4;
        dec_alu_sourcea_mux_sel = USEQ_SEL_ALUA_RFA;
        dec_alu_op_mux_sel = USEQ_SEL_ALUOP_AND;
        dec_flow_halt_mux_sel = USEQ_FLOW_HALT_NO;
        dec_pc_next_mux_sel = USEQ_SEL_PC_INCREMENT;
    end
    USEQ_OPCODES_MFSR: begin
        _codeop_ascii_r = "USEQ_OPCODES_MFSR";
        dec_writeback_dest_mux_sel = USEQ_SEL_DEST_RFC;
        dec_writeback_source_mux_sel = USEQ_SEL_WBSOURCE_ALU;
        dec_rf_sela_mux_sel = USEQ_SEL_RFA_R0;
        dec_rf_selb_mux_sel = USEQ_SEL_RFB_RS1;
        dec_rf_selc_mux_sel = USEQ_SEL_RFC_RD;
        dec_alu_sourceb_mux_sel = USEQ_SEL_ALUB_SR;
        dec_alu_sourcea_mux_sel = USEQ_SEL_ALUA_RFA;
        dec_alu_op_mux_sel = USEQ_SEL_ALUOP_ADD;
        dec_flow_halt_mux_sel = USEQ_FLOW_HALT_NO;
        dec_pc_next_mux_sel = USEQ_SEL_PC_INCREMENT;
    end
    USEQ_OPCODES_MTSR: begin
        _codeop_ascii_r = "USEQ_OPCODES_MTSR";
        dec_writeback_dest_mux_sel = USEQ_SEL_DEST_SR;
        dec_writeback_source_mux_sel = USEQ_SEL_WBSOURCE_ALU;
        dec_rf_sela_mux_sel = USEQ_SEL_RFA_R0;
        dec_rf_selb_mux_sel = USEQ_SEL_RFB_RS1;
        dec_rf_selc_mux_sel = USEQ_SEL_RFC_RD;
        dec_alu_sourceb_mux_sel = USEQ_SEL_ALUB_RFB;
        dec_alu_sourcea_mux_sel = USEQ_SEL_ALUA_RFA;
        dec_alu_op_mux_sel = USEQ_SEL_ALUOP_ADD;
        dec_flow_halt_mux_sel = USEQ_FLOW_HALT_NO;
        dec_pc_next_mux_sel = USEQ_SEL_PC_INCREMENT;
    end
    USEQ_OPCODES_ORI4: begin
        _codeop_ascii_r = "USEQ_OPCODES_ORI4";
        dec_writeback_dest_mux_sel = USEQ_SEL_DEST_RFC;
        dec_writeback_source_mux_sel = USEQ_SEL_WBSOURCE_ALU;
        dec_rf_sela_mux_sel = USEQ_SEL_RFA_R0;
        dec_rf_selb_mux_sel = USEQ_SEL_RFB_RS1;
        dec_rf_selc_mux_sel = USEQ_SEL_RFC_RD;
        dec_alu_sourceb_mux_sel = USEQ_SEL_ALUB_RFB_OR_UK4;
        dec_alu_sourcea_mux_sel = USEQ_SEL_ALUA_RFA;
        dec_alu_op_mux_sel = USEQ_SEL_ALUOP_OR;
        dec_flow_halt_mux_sel = USEQ_FLOW_HALT_NO;
        dec_pc_next_mux_sel = USEQ_SEL_PC_INCREMENT;
    end
    USEQ_OPCODES_SRAI: begin
        _codeop_ascii_r = "USEQ_OPCODES_SRAI";
        dec_writeback_dest_mux_sel = USEQ_SEL_DEST_RFC;
        dec_writeback_source_mux_sel = USEQ_SEL_WBSOURCE_ALU;
        dec_rf_sela_mux_sel = USEQ_SEL_RFA_RD;
        dec_rf_selb_mux_sel = USEQ_SEL_RFB_R1;
        dec_rf_selc_mux_sel = USEQ_SEL_RFC_RD;
        dec_alu_sourceb_mux_sel = USEQ_SEL_ALUB_UK4;
        dec_alu_sourcea_mux_sel = USEQ_SEL_ALUA_RFA;
        dec_alu_op_mux_sel = USEQ_SEL_ALUOP_SRA;
        dec_flow_halt_mux_sel = USEQ_FLOW_HALT_NO;
        dec_pc_next_mux_sel = USEQ_SEL_PC_INCREMENT;
    end
    USEQ_OPCODES_SEXT: begin
        _codeop_ascii_r = "USEQ_OPCODES_SEXT";
        dec_writeback_dest_mux_sel = USEQ_SEL_DEST_RFC;
        dec_writeback_source_mux_sel = USEQ_SEL_WBSOURCE_ALU;
        dec_rf_sela_mux_sel = USEQ_SEL_RFA_RD;
        dec_rf_selb_mux_sel = USEQ_SEL_RFB_RS2;
        dec_rf_selc_mux_sel = USEQ_SEL_RFC_RD;
        dec_alu_sourceb_mux_sel = USEQ_SEL_ALUB_RFB;
        dec_alu_sourcea_mux_sel = USEQ_SEL_ALUA_RFA;
        dec_alu_op_mux_sel = USEQ_SEL_ALUOP_SEXT;
        dec_flow_halt_mux_sel = USEQ_FLOW_HALT_NO;
        dec_pc_next_mux_sel = USEQ_SEL_PC_INCREMENT;
    end
    USEQ_OPCODES_BR: begin
        _codeop_ascii_r = "USEQ_OPCODES_BR";
        dec_writeback_dest_mux_sel = USEQ_SEL_DEST_COND_LINK_REG;
        dec_writeback_source_mux_sel = USEQ_SEL_WBSOURCE_ALU;
        dec_rf_sela_mux_sel = USEQ_SEL_RFA_R0;
        dec_rf_selb_mux_sel = USEQ_SEL_RFB_RS1;
        dec_rf_selc_mux_sel = USEQ_SEL_RFC_R0;
        dec_alu_sourceb_mux_sel = USEQ_SEL_ALUB_RFB_OR_UK4;
        dec_alu_sourcea_mux_sel = USEQ_SEL_ALUA_RFA;
        dec_alu_op_mux_sel = USEQ_SEL_ALUOP_ADD;
        dec_flow_halt_mux_sel = USEQ_FLOW_HALT_NO;
        dec_pc_next_mux_sel = USEQ_SEL_PC_COND_LOAD_FROM_ALU;
    end
    USEQ_OPCODES_XOR: begin
        _codeop_ascii_r = "USEQ_OPCODES_XOR";
        dec_writeback_dest_mux_sel = USEQ_SEL_DEST_RFC;
        dec_writeback_source_mux_sel = USEQ_SEL_WBSOURCE_ALU;
        dec_rf_sela_mux_sel = USEQ_SEL_RFA_RD;
        dec_rf_selb_mux_sel = USEQ_SEL_RFB_RS2;
        dec_rf_selc_mux_sel = USEQ_SEL_RFC_RD;
        dec_alu_sourceb_mux_sel = USEQ_SEL_ALUB_RFB;
        dec_alu_sourcea_mux_sel = USEQ_SEL_ALUA_RFA;
        dec_alu_op_mux_sel = USEQ_SEL_ALUOP_XOR;
        dec_flow_halt_mux_sel = USEQ_FLOW_HALT_NO;
        dec_pc_next_mux_sel = USEQ_SEL_PC_INCREMENT;
    end
    USEQ_OPCODES_SRA: begin
        _codeop_ascii_r = "USEQ_OPCODES_SRA";
        dec_writeback_dest_mux_sel = USEQ_SEL_DEST_RFC;
        dec_writeback_source_mux_sel = USEQ_SEL_WBSOURCE_ALU;
        dec_rf_sela_mux_sel = USEQ_SEL_RFA_RD;
        dec_rf_selb_mux_sel = USEQ_SEL_RFB_RS2;
        dec_rf_selc_mux_sel = USEQ_SEL_RFC_RD;
        dec_alu_sourceb_mux_sel = USEQ_SEL_ALUB_RFB;
        dec_alu_sourcea_mux_sel = USEQ_SEL_ALUA_RFA;
        dec_alu_op_mux_sel = USEQ_SEL_ALUOP_SRA;
        dec_flow_halt_mux_sel = USEQ_FLOW_HALT_NO;
        dec_pc_next_mux_sel = USEQ_SEL_PC_INCREMENT;
    end
    USEQ_OPCODES_ADD3: begin
        _codeop_ascii_r = "USEQ_OPCODES_ADD3";
        dec_writeback_dest_mux_sel = USEQ_SEL_DEST_RFC;
        dec_writeback_source_mux_sel = USEQ_SEL_WBSOURCE_ALU;
        dec_rf_sela_mux_sel = USEQ_SEL_RFA_RS1;
        dec_rf_selb_mux_sel = USEQ_SEL_RFB_RS2;
        dec_rf_selc_mux_sel = USEQ_SEL_RFC_RD;
        dec_alu_sourceb_mux_sel = USEQ_SEL_ALUB_RFB;
        dec_alu_sourcea_mux_sel = USEQ_SEL_ALUA_RFA;
        dec_alu_op_mux_sel = USEQ_SEL_ALUOP_ADD;
        dec_flow_halt_mux_sel = USEQ_FLOW_HALT_NO;
        dec_pc_next_mux_sel = USEQ_SEL_PC_INCREMENT;
    end
    USEQ_OPCODES_ADD2: begin
        _codeop_ascii_r = "USEQ_OPCODES_ADD2";
        dec_writeback_dest_mux_sel = USEQ_SEL_DEST_RFC;
        dec_writeback_source_mux_sel = USEQ_SEL_WBSOURCE_ALU;
        dec_rf_sela_mux_sel = USEQ_SEL_RFA_RD;
        dec_rf_selb_mux_sel = USEQ_SEL_RFB_RS1;
        dec_rf_selc_mux_sel = USEQ_SEL_RFC_RD;
        dec_alu_sourceb_mux_sel = USEQ_SEL_ALUB_RFB_OR_UK4;
        dec_alu_sourcea_mux_sel = USEQ_SEL_ALUA_RFA;
        dec_alu_op_mux_sel = USEQ_SEL_ALUOP_ADD;
        dec_flow_halt_mux_sel = USEQ_FLOW_HALT_NO;
        dec_pc_next_mux_sel = USEQ_SEL_PC_INCREMENT;
    end
    USEQ_OPCODES_SRLI: begin
        _codeop_ascii_r = "USEQ_OPCODES_SRLI";
        dec_writeback_dest_mux_sel = USEQ_SEL_DEST_RFC;
        dec_writeback_source_mux_sel = USEQ_SEL_WBSOURCE_ALU;
        dec_rf_sela_mux_sel = USEQ_SEL_RFA_RD;
        dec_rf_selb_mux_sel = USEQ_SEL_RFB_R1;
        dec_rf_selc_mux_sel = USEQ_SEL_RFC_RD;
        dec_alu_sourceb_mux_sel = USEQ_SEL_ALUB_UK4;
        dec_alu_sourcea_mux_sel = USEQ_SEL_ALUA_RFA;
        dec_alu_op_mux_sel = USEQ_SEL_ALUOP_SRL;
        dec_flow_halt_mux_sel = USEQ_FLOW_HALT_NO;
        dec_pc_next_mux_sel = USEQ_SEL_PC_INCREMENT;
    end
    USEQ_OPCODES_SW: begin
        _codeop_ascii_r = "USEQ_OPCODES_SW";
        dec_writeback_dest_mux_sel = USEQ_SEL_DEST_MEM16;
        dec_writeback_source_mux_sel = USEQ_SEL_WBSOURCE_RFB;
        dec_rf_sela_mux_sel = USEQ_SEL_RFA_RS1;
        dec_rf_selb_mux_sel = USEQ_SEL_RFB_RS1;
        dec_rf_selc_mux_sel = USEQ_SEL_RFC_RD;
        dec_alu_sourceb_mux_sel = USEQ_SEL_ALUB_K4;
        dec_alu_sourcea_mux_sel = USEQ_SEL_ALUA_RFA;
        dec_alu_op_mux_sel = USEQ_SEL_ALUOP_ADD;
        dec_flow_halt_mux_sel = USEQ_FLOW_HALT_NO;
        dec_pc_next_mux_sel = USEQ_SEL_PC_INCREMENT;
    end
    USEQ_OPCODES_SRL: begin
        _codeop_ascii_r = "USEQ_OPCODES_SRL";
        dec_writeback_dest_mux_sel = USEQ_SEL_DEST_RFC;
        dec_writeback_source_mux_sel = USEQ_SEL_WBSOURCE_ALU;
        dec_rf_sela_mux_sel = USEQ_SEL_RFA_RD;
        dec_rf_selb_mux_sel = USEQ_SEL_RFB_RS2;
        dec_rf_selc_mux_sel = USEQ_SEL_RFC_RD;
        dec_alu_sourceb_mux_sel = USEQ_SEL_ALUB_RFB;
        dec_alu_sourcea_mux_sel = USEQ_SEL_ALUA_RFA;
        dec_alu_op_mux_sel = USEQ_SEL_ALUOP_SRL;
        dec_flow_halt_mux_sel = USEQ_FLOW_HALT_NO;
        dec_pc_next_mux_sel = USEQ_SEL_PC_INCREMENT;
    end
    USEQ_OPCODES_SLL: begin
        _codeop_ascii_r = "USEQ_OPCODES_SLL";
        dec_writeback_dest_mux_sel = USEQ_SEL_DEST_RFC;
        dec_writeback_source_mux_sel = USEQ_SEL_WBSOURCE_ALU;
        dec_rf_sela_mux_sel = USEQ_SEL_RFA_RD;
        dec_rf_selb_mux_sel = USEQ_SEL_RFB_RS2;
        dec_rf_selc_mux_sel = USEQ_SEL_RFC_RD;
        dec_alu_sourceb_mux_sel = USEQ_SEL_ALUB_RFB;
        dec_alu_sourcea_mux_sel = USEQ_SEL_ALUA_RFA;
        dec_alu_op_mux_sel = USEQ_SEL_ALUOP_SLL;
        dec_flow_halt_mux_sel = USEQ_FLOW_HALT_NO;
        dec_pc_next_mux_sel = USEQ_SEL_PC_INCREMENT;
    end
    USEQ_OPCODES_SLLI: begin
        _codeop_ascii_r = "USEQ_OPCODES_SLLI";
        dec_writeback_dest_mux_sel = USEQ_SEL_DEST_RFC;
        dec_writeback_source_mux_sel = USEQ_SEL_WBSOURCE_ALU;
        dec_rf_sela_mux_sel = USEQ_SEL_RFA_RD;
        dec_rf_selb_mux_sel = USEQ_SEL_RFB_R1;
        dec_rf_selc_mux_sel = USEQ_SEL_RFC_RD;
        dec_alu_sourceb_mux_sel = USEQ_SEL_ALUB_UK4;
        dec_alu_sourcea_mux_sel = USEQ_SEL_ALUA_RFA;
        dec_alu_op_mux_sel = USEQ_SEL_ALUOP_SLL;
        dec_flow_halt_mux_sel = USEQ_FLOW_HALT_NO;
        dec_pc_next_mux_sel = USEQ_SEL_PC_INCREMENT;
    end
    USEQ_OPCODES_SB: begin
        _codeop_ascii_r = "USEQ_OPCODES_SB";
        dec_writeback_dest_mux_sel = USEQ_SEL_DEST_MEM8;
        dec_writeback_source_mux_sel = USEQ_SEL_WBSOURCE_RFB;
        dec_rf_sela_mux_sel = USEQ_SEL_RFA_RS1;
        dec_rf_selb_mux_sel = USEQ_SEL_RFB_RS1;
        dec_rf_selc_mux_sel = USEQ_SEL_RFC_RD;
        dec_alu_sourceb_mux_sel = USEQ_SEL_ALUB_K4;
        dec_alu_sourcea_mux_sel = USEQ_SEL_ALUA_RFA;
        dec_alu_op_mux_sel = USEQ_SEL_ALUOP_ADD;
        dec_flow_halt_mux_sel = USEQ_FLOW_HALT_NO;
        dec_pc_next_mux_sel = USEQ_SEL_PC_INCREMENT;
    end
    USEQ_OPCODES_OR: begin
        _codeop_ascii_r = "USEQ_OPCODES_OR";
        dec_writeback_dest_mux_sel = USEQ_SEL_DEST_RFC;
        dec_writeback_source_mux_sel = USEQ_SEL_WBSOURCE_ALU;
        dec_rf_sela_mux_sel = USEQ_SEL_RFA_RD;
        dec_rf_selb_mux_sel = USEQ_SEL_RFB_RS2;
        dec_rf_selc_mux_sel = USEQ_SEL_RFC_RD;
        dec_alu_sourceb_mux_sel = USEQ_SEL_ALUB_RFB;
        dec_alu_sourcea_mux_sel = USEQ_SEL_ALUA_RFA;
        dec_alu_op_mux_sel = USEQ_SEL_ALUOP_OR;
        dec_flow_halt_mux_sel = USEQ_FLOW_HALT_NO;
        dec_pc_next_mux_sel = USEQ_SEL_PC_INCREMENT;
    end
   //@end[case_inst]
   endcase
  end


  wire cond_branch;
  assign cond_branch = 1'b1; // Fixme
   
  
  always @* begin
  if(halt | halt_r) begin
      npc = pc_r;      
    end 
	else begin
	
    case(dec_pc_next_mux_sel)
      USEQ_SEL_PC_INCREMENT: begin
         npc <= pc_r + USEQ_PC_INC;
      end
      USEQ_SEL_PC_FROM_ALU: begin
         npc <= dest_bus;
      end
      USEQ_SEL_PC_COND_LOAD_FROM_ALU: begin
        if(cond_branch) begin
         npc <= dest_bus;
        end
        else begin
          npc <= pc_r + USEQ_PC_INC;
        end
      end

      default: begin
        npc <= pc_r + USEQ_PC_INC;
      end
      
    endcase
end	
  end
  
  

  
  
  // Program counter management
/*   always @* begin
    if(halt | halt_r) begin
      npc = pc_r;      
    end  
    else if (write_to_pc) begin
      npc <= dest_bus;        
    end
    else begin      
      npc <= pc_r + USEQ_PC_INC;
    end */


  //end

  always @(ram_req or negedge rst_n) begin
    if(rst_n == 1'b0) begin
      /*AUTORESET*/
      // Beginning of autoreset for uninitialized flops
      pc_r <= {(1+(USEQ_WORD_MSB)){1'b0}};
      // End of automatics
   rf_req<=0 ;
   cpu_ack <=0 ;
    end
    else begin
      pc_r <= npc;
      halt_r <= halt;      
    end
  end // always @ (posedge clk or negedge rst_n)


  assign carry_in = 0; // FIXME
  
  
  //RAM Read
  
  
  pulse_generator PG1(ram_clk,ram_req);
  always @(cpu_req or rf_ack or negedge rst_n) begin
	if(rst_n == 1'b0) begin
		ram_req<=0;
	end
	else begin
  		ram_req <= !ram_req;
	end  
  end
 
 wire ram_req_delayed;

 delay_cell #(.DELAY (10)) D1 (ram_req_delayed,ram_req);
  always @(ram_req_delayed or negedge rst_n) begin
	if(rst_n == 1'b0) begin
		ram_ack <= 0;
	end
	else begin
		ram_ack <= !ram_ack;
	end
  end

always @(rf_ack or negedge rst_n) begin
	if(rst_n == 1'b0) begin
		ram_req <= 0;
	end
	else begin			
	ram_req <= !ram_req;
	end
end

 wire rf_req_delayed;

delay_cell #(.DELAY (10)) D2 (rf_req_delayed,rf_req);
  always @(rf_req_delayed or negedge rst_n) begin
	if(rst_n == 1'b0) begin
		rf_ack <= 0;
	end
	else begin
		rf_ack <= !rf_ack;
	end
  end
  
  wire rf_clk;
  pulse_generator PG1(rf_clk,rf_req);
  always @(cpu_req or alu_ack or negedge rst_n) begin
	if(rst_n == 1'b0) begin
		rf_req<=0;
	end
	else begin
  		rf_req <= !rf_req;
	end  
  end
  
  simd_useq_alu U_ALU (
                       // Outputs
                       .c               (alu_c[USEQ_WORD_MSB:0]),
			.alu_ack	(alu_ack),
                       // Inputs
			.alu_req	(alu_req),
                       .a               (alu_a[USEQ_WORD_MSB:0]),
                       .b               (alu_b[USEQ_WORD_MSB:0]),
                       .op              (dec_alu_op_mux_sel[USEQ_SEL_ALUOP_MSB:0]),
                       .carry_in        (carry_in),
                       .carry_out       (carry_out),
			.rst_n		(rst_n));
  
  

   /* simd_useq_rf AUTO_TEMPLATE(
    ); */
  simd_useq_rf U_RF (
                     .rf_a              (rf_a[USEQ_WORD_MSB:0]),
                     .rf_b              (rf_b[USEQ_WORD_MSB:0]),
                     /*AUTOINST*/
                     // Inputs
                     .rf_sela           (rf_sela[USEQ_RF_SEL_MSB:0]),
                     .rf_selb           (rf_selb[USEQ_RF_SEL_MSB:0]),
                     .rf_selc           (rf_selc[USEQ_RF_SEL_MSB:0]),
                     .rf_c              (rf_c[USEQ_WORD_MSB:0]),
                     .rf_write          (rf_write),
                     .clk               (rf_clk),
                     .rst_n             (rst_n));
  
  

 
endmodule // simd_useq
/*  
 Local Variables:
 verilog-library-directories:(
 "."
 )
 End:
 */
