module simd_useq_alu (/*AUTOARG*/
  // Outputs
  c,alu_ack,
  // Inputs
  alu_req, a, b, op, carry_in, carry_out,rst_n
  );
  
`include "simd_params.v"

  // ALU data port
  input [USEQ_WORD_MSB:0] a; 
  input [USEQ_WORD_MSB:0] b;
  output[USEQ_WORD_MSB:0] c;

  input [USEQ_SEL_ALUOP_MSB:0] op;
  input                        carry_in;
  input                        carry_out;
  input							alu_req;
  input				rst_n;
  output 						alu_ack;

    


  /*AUTOINPUT*/
  /*AUTOOUTPUT*/
  
  /*AUTOREG*/
  // Beginning of automatic regs (for this module's undeclared outputs)
  reg [USEQ_WORD_MSB:0] c;
  // End of automatics
  /*AUTOWIRE*/


  
  reg          force_carry;
  reg          force_carry_value;
reg alu_ack;

  reg  [USEQ_WORD_MSB:0] adder_a; 
  reg  [USEQ_WORD_MSB:0] adder_b;
  wire [USEQ_WORD_MSB:0]  adder_res;
  wire                   adder_cin;
  wire			alu_req_delayed;
  
  
  delay_cell D1(alu_req_delayed, alu_req );
  always @(posedge alu_req_delayed or negedge rst_n) begin
	if(rst_n == 1'b0) begin
		alu_ack <= 0;
	end
	else begin
  		alu_ack <= 1;
	end
  end

  
  always @* begin
    adder_a = a;
    adder_b = b;
    c = adder_res;    
    force_carry       =1'b0;
    force_carry_value =1'b0;
    case(op)
      USEQ_SEL_ALUOP_ADD: begin
        force_carry        = 1'b1;
        force_carry_value  = 1'b0;
        c = adder_res;    
      end
      USEQ_SEL_ALUOP_SRA: begin
        c = a >>> b[3:0];
      end
      USEQ_SEL_ALUOP_SRL: begin
        c = a >> b[3:0];
      end
      USEQ_SEL_ALUOP_SLL: begin
        c = a << b[3:0];
      end
      USEQ_SEL_ALUOP_AND: begin
        c = a & b;
      end
      USEQ_SEL_ALUOP_OR: begin
        c = a | b;
      end
      USEQ_SEL_ALUOP_XOR: begin
        c = a ^ b;
      end
      USEQ_SEL_ALUOP_XNOR: begin
        c = ~(a ^ b);
      end
      USEQ_SEL_ALUOP_SEXT: begin
        c = { {(USEQ_HALF_WORD_BITS){b[USEQ_HALF_WORD_BITS]}},b[USEQ_HALF_WORD_BITS-1:0]};
      end
      default:begin
        adder_a = a;
        adder_b = b;
        c = adder_res;    
        force_carry       =1'b0;
        force_carry_value =1'b0;
      end
    endcase // case (op)    
  end
  
  


  assign is_not_equal = (|adder_res);
  assign is_equal = !is_not_equal;
  assign is_less = adder_res[USEQ_WORD_MSB];
  assign is_greater = !adder_res[USEQ_WORD_MSB] && is_not_equal;

  assign is_less_or_equal = is_less || is_equal;
  assign is_greater_or_equal = is_greater || is_equal;

  
  assign adder_cin  = force_carry ? force_carry_value : carry_in;
  assign carry_cout = adder_cout;
 
  simd_generic_adder 
    #(.WIDTH(USEQ_WORD_BITS)) 
  U_ALU_ADDER (             
               // Outputs
               .res              (adder_res[USEQ_WORD_BITS-1:0]),
               .cout             (adder_cout),
               // Inputs
               .a                (adder_a[USEQ_WORD_BITS-1:0]),
               .b                (adder_b[USEQ_WORD_BITS-1:0]),
               .cin              (adder_cin));
  
  

  
endmodule // simd_useq_alu
/*  
 Local Variables:
 verilog-library-directories:(
 "."
 )
 End:
 */
