#Read dc_setup
source ../synt/dc_setup.tcl

#Setup libs
#Get just CSC + SC lib from L7
set link_library "* ../lib/import/RFO_SC_S9SCLIB_N1/db/rfo_sc_s9sclib_n1-NOM1V20T25+nldm.db"

#Limit some noisy messages
suppress_message {CMD-041 RC-004 RC-005 RC-008 RC-009}
set_message_info -id PTE-060 -limit 5

p_read_env {QUIT SAIF VCD READ} false
p_read_env {TARGET} pt

#From common_setup.tcl
set top "module_wrapper"

set RESULTS_DIR "../ntl"

set VCD_INSTANCE "tb/U_DUT"


#Read netlist
if {$read == "cts"} {
    #Read anetlist + spef from FE
    read_verilog ${RESULTS_DIR}/${top}.cts.v
    link ${top}
    
    #Read spef + save sbpf. Use sbpf if existing since its faster.
    #Watch the log to make sure every net is annotated
    set spef_file ${RESULTS_DIR}/${top}.cts.spef.gz
    set sbpf_file ${RESULTS_DIR}/${top}.cts.sbpf
    
    if [file exists $sbpf_file] {
        read_parasitics -format SBPF $sbpf_file
    } else {
        read_parasitics -triplet_type max $spef_file
        write_parasitics -format SBPF $sbpf_file
    }
} elseif {$read == "synt"} {
    #Read a netlist from DC, uses wireload models
    read_verilog ${RESULTS_DIR}/${top}.mapped.v
    link ${top}
} 

#Read SDC to get clock information
read_sdc ${RESULTS_DIR}/${top}.mapped.sdc

set power_enable_analysis 1

if {$read == "cts"} {
    set_propagated_clock [all_clocks]
}

#Read saif / VCD
if {$saif} {
    set power_analysis_mode averaged
    read_saif ../sim/tb/vcdplus.saif  -strip_path $VCD_INSTANCE
}

if {$vcd} {
    set power_analysis_mode time_based

    #Create a power waveform. Interval is 1000ns
    set_power_analysis_options -waveform_output $target -waveform_interval 1000

    read_vcd ../sim/tb/vcdplus.vcd.gz -strip_path $VCD_INSTANCE

}

#Report the switching activity annotation this should be 100% for ntl
#For RTL simulation all sequential elements should be annotated
report_switching_activity

# Other settings
set pwr_default_voltage 1.8

update_power



#Report some checks
check_power
check_power -override_defaults missing_table -verbose

#If no clocktree try to estimate clocktree power
#However the correlation may not be good
#if {$read == "synt"} {
#    estimate_clock_network_power SClib-typ+ind/bufbd7
#}

#Report power
redirect -tee -file ${REPORTS_DIR}/${top}.${target}.rpt {
    report_power
}

#Save PT session for later debug
save_session $target.session

#Quit if QUIT was defined.
if {[is_true $quit]} {
    quit
}

