
//****************************************************************************
//                 PROPRIETARY INFORMATION OF ATMEL CORPORATION
//
//                     32-bit AVR UC3 / SAM source code file.
//                       Copyright Atmel Corporation
//
//         The scope of use of the material contained herein is limited
//         strictly to the terms of the LICENSE which grants permission
//         for the use of the material.  Distribution of this material
//            and all copies thereof to any entity other than to the
//           EMPLOYEES of the Licensee, as the term is defined in the
//        LICENSE, is strictly forbidden without a prior written consent
//                            of Atmel Corporation.
//****************************************************************************
//****************************************************************************
//
//  Project number  :  primitives_110n_U1004
//  Module name     :  primitive_latch_set_reset
//  Description     :  primitive gates for 110n HVT
//
//  URL             :  $URL: http://norsvn01.norway.atmel.com/ic/avr32/ultra/flow/primitives_110n_U1004/version/101/rtl/module/verilog/primitive_latch_set_reset.v $
//  Author          :  $Author: teleistad $
//  Date            :  $Date: 2013-12-12 16:19:38 +0100 (Thu, 12 Dec 2013) $
//  Revision        :  $Revision: 215732 $
//
//****************************************************************************



`ifndef DEFAULT_LIB
  `define DEFAULT_LIB "S8"
`endif
module primitive_latch_set_reset #(parameter LIB=`DEFAULT_LIB)
  (
  output z, zn,
  input set_n, clear, rst_n
  );

  wire z0 ;
`ifdef PRIMITIVE_TIMING
  assign #0.1 z = z0 ;
`else
  assign z = z0 ;
`endif

  wire  set;
  wire rst_n_qual;
  wire clear_n;
  
  // Function table:
  // cdn sdn  e  d  q  qn
  //  0   0   x  x  1  1
  //  0   1   x  x  0  1
  //  1   0   x  x  1  0
  //  1   1   0  x  q  qn
  //  1   1   1  0  0  1
  //  1   1   1  1  1  0

  if (LIB=="S10X3") begin:U
      inv0d0x3s10 INVA (.i(set_n), .zn(set));
      inv0d0x3s10 INVB (.i(clear), .zn(clear_n));
      nd02d1x3s10 NAND2 (.a1(clear_n), .a2(rst_n), .zn(rst_n_qual));
      srld1x3s10 LATCH (.Q(z0), .S(set), .R(rst_n_qual));
      inv0d0x3s10 INVC (.i(z), .zn(zn));
  end

  else if (LIB=="S8") begin:U
      invd3s8 INVA (.I(set_n), .O(set));
      invd3s8 INVB (.I(clear), .O(clear_n));
      nd2d3s8 NAND2 (.I1(clear_n), .I2(rst_n), .O(rst_n_qual));
      srld1s8 LATCH (.Q(z0), .S(set), .R(rst_n_qual));
      invd3s8 INVC (.I(z), .O(zn));
  end

  else if (LIB=="S8FSHVT") begin:U
      invd3s8fshvt INVA (.I(set_n), .O(set));
      invd3s8fshvt INVB (.I(clear), .O(clear_n));
      nd2d3s8fshvt NAND2 (.I1(clear_n), .I2(rst_n), .O(rst_n_qual));
      srld1s8fshvt LATCH (.Q(z0), .S(set), .R(rst_n_qual));
      invd3s8fshvt INVC (.I(z), .O(zn));
  end

  else if (LIB=="S8FHVT") begin:U
      invd3s8fhvt INVA (.I(set_n), .O(set));
      invd3s8fhvt INVB (.I(clear), .O(clear_n));
      nd2d3s8fhvt NAND2 (.I1(clear_n), .I2(rst_n), .O(rst_n_qual));
      srld1s8fhvt LATCH (.Q(z0), .S(set), .R(rst_n_qual));
      invd3s8fhvt INVC (.I(z), .O(zn));
  end

  
endmodule // primitive_latch_set_reset
