//****************************************************************************
//                 PROPRIETARY INFORMATION OF ATMEL CORPORATION
//
//                     AVR-II(R) RISC source code file.
//                       Copyright Atmel Corporation
//                                       
//         The scope of use of the material contained herein is limited
//         strictly to the terms of the LICENSE which grants permission
//         for the use of the material.  Distribution of this material
//            and all copies thereof to any entity other than to the
//           EMPLOYEES of the Licensee, as the term is defined in the
//        LICENSE, is strictly forbidden without a prior written consent
//                            of Atmel Corporation.
//****************************************************************************
//****************************************************************************
// 
//  A V R - I I
//  RTL IMPLEMENTATION, STILETTO PLATFORM
//
//  Filename        :  amba_params.v 
//  Revision        :  $Revision: 137963 $
//  Checkin Date    :  $Date: 2012-09-26 12:26:26 +0200 (Wed, 26 Sep 2012) $
//  Last checkin by :  $Author: rabjoernerud $
//  CVS State       :  $State$ 
//  
//  Project number  :  I7091
//  Module name     :  RTL CPU Pipeline Code
//  Description     :  Holds all common AMBA AHB constants
//
//  Author          :  erenno
//  Date            :  Wed Jan 12 15:17:57 2005
//
//  Modification history at bottom of this file
//
//****************************************************************************

`ifndef AMBA_PARAMS_SV
`define AMBA_PARAMS_SV

parameter HADDR_BITS = 32,
          HDATA_BITS = 32;
parameter HADDR_MSB = HADDR_BITS - 1;
parameter HDATA_MSB = HDATA_BITS - 1;

//HTRANS values
parameter HTRANS_BITS   = 2;
parameter HTRANS_MSB    = HTRANS_BITS - 1;

parameter HTRANS_IDLE   = 2'b00;
parameter HTRANS_BUSY   = 2'b01;
parameter HTRANS_NONSEQ = 2'b10;
parameter HTRANS_SEQ    = 2'b11;


//HBURST values
parameter HBURST_BITS   = 3;
parameter HBURST_MSB    = HBURST_BITS - 1;

parameter HBURST_SINGLE = 3'b000;
parameter HBURST_INCR   = 3'b001;
parameter HBURST_WRAP4  = 3'b010;
parameter HBURST_INCR4  = 3'b011;
parameter HBURST_WRAP8  = 3'b100;
parameter HBURST_INCR8  = 3'b101;
parameter HBURST_WRAP16 = 3'b110;
parameter HBURST_INCR16 = 3'b111;


//HSIZE values (AMBA spec has 3 bits here, allowing for 1024-bit accesses!)
// And we should also have 3 bits here, because all IPs use 3 bit HSIZE
parameter HSIZE_BITS       = 3;
parameter HSIZE_MSB        = HSIZE_BITS - 1;

parameter HSIZE_BYTE       = 3'b000;
parameter HSIZE_HALFWORD   = 3'b001;
parameter HSIZE_WORD       = 3'b010;
parameter HSIZE_DOUBLEWORD = 3'b011;

parameter HSIZE_8          = HSIZE_BYTE;
parameter HSIZE_16         = HSIZE_HALFWORD;
parameter HSIZE_32         = HSIZE_WORD;
parameter HSIZE_64         = HSIZE_DOUBLEWORD;

parameter HSIZE_HWORD      = HSIZE_HALFWORD;
parameter HSIZE_DWORD      = HSIZE_DOUBLEWORD;


//HPROT values
parameter HPROT_BITS           = 4;
parameter HPROT_MSB            = HPROT_BITS - 1;

parameter HPROT_CACHEABLE_BIT  = 3;
parameter HPROT_BUFFERABLE_BIT = 2;
parameter HPROT_PRIVILEGED_BIT = 1;
parameter HPROT_DATA_BIT       = 0;

parameter HPROT_C_BIT          = HPROT_CACHEABLE_BIT;
parameter HPROT_B_BIT          = HPROT_BUFFERABLE_BIT;
parameter HPROT_P_BIT          = HPROT_PRIVILEGED_BIT;
parameter HPROT_D_BIT          = HPROT_DATA_BIT;

parameter HPROT_CACHEABLE      = 4'b1000;
parameter HPROT_UNCACHEABLE    = 4'b0000;
parameter HPROT_BUFFERABLE     = 4'b0100;
parameter HPROT_UNBUFFERABLE   = 4'b0000;
parameter HPROT_PRIVILEGED     = 4'b0010;
parameter HPROT_UNPRIVILEGED   = 4'b0000;
parameter HPROT_DATA           = 4'b0001;
parameter HPROT_OPCODE         = 4'b0000;

parameter HPROT_C              = HPROT_CACHEABLE;
parameter HPROT_UC             = HPROT_UNCACHEABLE;
parameter HPROT_B              = HPROT_BUFFERABLE;
parameter HPROT_UB             = HPROT_UNBUFFERABLE;
parameter HPROT_P              = HPROT_PRIVILEGED;
parameter HPROT_UP             = HPROT_UNPRIVILEGED;
parameter HPROT_D              = HPROT_DATA;
parameter HPROT_O              = HPROT_OPCODE;
parameter HPROT_I              = HPROT_OPCODE;


//HRESP values
parameter HRESP_BITS  = 2;
parameter HRESP_MSB   = HRESP_BITS - 1;
parameter HRESP_OKAY  = 2'b00;
parameter HRESP_ERROR = 2'b01;
parameter HRESP_RETRY = 2'b10;
parameter HRESP_SPLIT = 2'b11;

parameter HRESP_OK    = HRESP_OKAY;

//APB values
parameter PDATA_BITS = 32;
parameter PDATA_WIDTH = PDATA_BITS;
parameter PADDR_BITS = 6'd32;
parameter PADDR_WIDTH = PADDR_BITS;
parameter PADDR_MSB  = PADDR_BITS - 1;
parameter PDATA_MSB  = PDATA_BITS - 1;

parameter PSTRB_WIDTH = (PDATA_WIDTH+7)/8; // 1 strobe bit per byte

//PSTRB values
parameter PSTRB_BITS = 4;
parameter PSTRB_MSB  = PSTRB_BITS - 1;

//PPROT values
parameter PPROT_BITS = 3;
parameter PPROT_WIDTH = PPROT_BITS;
parameter PPROT_MSB  = PPROT_BITS - 1;

//**************************************************************************
// 
// Modification History 
// -------------------------------------------------------------------------
//
// $Log$
// Revision 1.2  2005/08/19 16:41:35  erenno
// debugged after integration with hmatrix, hbridge and USART
//
// Revision 1.1  2005/01/13 17:47:22  erenno
// Initial revision
//
// 
//************************************************************************** 

`endif
