//****************************************************************************
//                 PROPRIETARY INFORMATION OF ATMEL CORPORATION
//
//                     32-bit AVR UC3 / SAM source code file.
//                       Copyright Atmel Corporation
//
//         The scope of use of the material contained herein is limited
//         strictly to the terms of the LICENSE which grants permission
//         for the use of the material.  Distribution of this material
//            and all copies thereof to any entity other than to the
//           EMPLOYEES of the Licensee, as the term is defined in the
//        LICENSE, is strictly forbidden without a prior written consent
//                            of Atmel Corporation.
//****************************************************************************
//****************************************************************************
//
//  Filename        :  ip_h.sv
//
//  Project number  :  -
//  Module name     :  -
//  Description     :  IP Functions for APB interface
//
//  Author          :  SG/TD/PM
//  Date            :  June 2012
//
//****************************************************************************

//**********************************************************************************
//                               IP FUNCTIONS
//**********************************************************************************

  // Returns position of 1st '1' in a vector ('b0011100 => 2)
  function automatic logic[4:0] f_get_lsb;
  input logic [31:0] in;
  integer i, found;
    begin
      // for(pos=0; !in[pos]; pos++); // makes FM crash
      found=0;
      f_get_lsb = 0;
      for(i=0; i<32; i++)
        if(in[i] && !found)
          begin
            found = 1;
            f_get_lsb = i;
          end
    end
  endfunction

  
  // Returns position of last '1' in a vector ('b0011100 => 4)
  function automatic logic[4:0] f_get_msb;
  input logic [31:0] in;
  integer i, found;
    begin
      // for(pos=0; !in[pos]; pos++); // makes FM crash
      found=0;
      f_get_msb = 0;
      for(i=31; i>=0; i--)
        if(in[i] && !found)
          begin
            found = 1;
            f_get_msb = i;
          end
    end
  endfunction


  // Returns the size of a mask ('b0011100 => 3)
  function automatic logic[4:0] f_get_size;
  input logic [31:0] in;
    begin
      f_get_size = f_get_msb(in) - f_get_lsb(in) + 1;
    end
  endfunction


  // Returns byte index according to reg offset
  function automatic integer f_byte_sel;
    input logic [1:0] reg_offset;
    begin
      case (reg_offset[1:0])
        0 : f_byte_sel = 0;
        1 : f_byte_sel = 8;
        2 : f_byte_sel = 16;
        3 : f_byte_sel = 24;
      endcase // case (reg_offset[1:0])
    end
  endfunction

  // Returns '1' when reg offset and (paddr + pstrb) match
  function automatic logic f_byte_lane_access;
    input logic [31:0] apb_paddr;
    input logic [3:0]  pstrb;
    input logic [31:0] reg_offset;
    
    begin
      if ((apb_paddr[31:2] == reg_offset[31:2]) &&
          (pstrb[reg_offset[1:0]] == 1))
        f_byte_lane_access = 1'b1;
      else
        f_byte_lane_access = 1'b0;
    end
  endfunction

  function automatic logic f_byte_lane_access_masked;
    input logic [31:0] apb_paddr;
    input logic [3:0]  pstrb;
    input logic [31:0] reg_offset;
    input logic [31:0] address_mask;
    
    begin
      f_byte_lane_access_masked = f_byte_lane_access(apb_paddr | address_mask, pstrb, reg_offset | address_mask);
    end
  endfunction

  function automatic logic[3:0] f_byte_lane_access_vector;
    input logic        apb_access;     
    input logic [31:0] apb_paddr;
    input logic [3:0]  pstrb;
    input logic [31:0] reg_offset;
    integer      i;
    logic [3:0]  tmp;
    
    begin
      for(i=0; i<4; i++)
        tmp[i] = apb_access && f_byte_lane_access(apb_paddr, pstrb, reg_offset+i);

      f_byte_lane_access_vector[3:0] = tmp[3:0];
       
    end
  endfunction

  function automatic logic[3:0] f_byte_lane_access_vector_masked;
    input logic        apb_access;     
    input logic [31:0] apb_paddr;
    input logic [3:0]  pstrb;
    input logic [31:0] reg_offset;
    input logic [31:0] address_mask;    
    begin

      f_byte_lane_access_vector_masked[3:0] = f_byte_lane_access_vector(apb_access, apb_paddr | address_mask, pstrb, reg_offset | address_mask);
       
    end
  endfunction
