
`timescale 1ns/1ps



module tb_simd (/*AUTOARG*/);
  
`include "simd_params.v"  

  

  
  /*AUTOINPUT*/
  
  /*AUTOOUTPUT*/
  
  /*AUTOREG*/
  
  wire                  clk;                    // From U_CLOK_GEN of clock_gen.v
  wire [USEQ_WORD_MSB:0] pc_r;                  // From U_SIMD of simd.v
  wire                  rst_n;                  // From U_RESET_GEN of reset_gen.v
  wire			rst_n_temp;
  // End of automatics
	
parameter RAM_SIZE = 1536;
  reg [USEQ_INST_MSB:0] instruction_reg;
  wire [USEQ_INST_MSB:0] instruction; 

  
	wire me;	
	wire re;
	reg [3:0] we;
	reg [3:0]rm;
	reg [15:0]instruction_in;
	reg pd;	
	reg pdd;
	reg [15:0]     mem0 [0:RAM_SIZE-1];
	reg load_program;
	reg started_prog;
	reg [10:0]count;
	reg [3:0] sel_mode;
   /* simd AUTO_TEMPLATE(
    .instruction             (instruction_reg[USEQ_INST_MSB:0]),
    ); */
	

  simd U_SIMD (
                          /*AUTOINST*/
               // Outputs
               .pc_r                    (pc_r[USEQ_WORD_MSB:0]),
               // Inputs
               .clk                     (clk),
               .instruction             (instruction_reg[USEQ_INST_MSB:0]), // Templated
               .rst_n                   (rst_n)
			   .sel_mode				(sel_mode[3:0]));
  
  
   /* ram AUTO_TEMPLATE(
    .addr                 (pc_r[USEQ_WORD_MSB:1]), // Fixme - truncation...
    .we                   (1'b0),
    .write_data           ({(USEQ_WORD_BITS){1'b0}}),
     .read_data            (instruction[USEQ_INST_MSB:0]),
    .clock                (clk),
    .cs                   (1'b1),
        ); */
  /*ram #(.WORD_WIDTH(USEQ_WORD_BITS),
        .ADDR_WIDTH(USEQ_WORD_BITS-1)) 
  U_USEQ_CODE_RAM (
                   .read_data           (instruction[USEQ_INST_MSB:0]),
                   // Inputs
                   .clock               (clk),                   // Templated
                   .addr                (pc_r[USEQ_WORD_MSB:1]), // Templated
                   .write_data          ({(USEQ_WORD_BITS){1'b0}}), // Templated
                   .cs                  (1'b1),                  // Templated
                   .we                  (1'b0));                  // Templated
				   
  
  */ prog_mem_wrapper PROGMEMWRAP (
	.load_program (load_program),
	.prog_clk (prog_clk),
	.prog_adr (count[10:0]),
	.clk	(clk),
	.pc_r	(pc_r[15:0]),
	.instruction_in	(instruction_in[15:0]),
	.instruction	(instruction[USEQ_INST_MSB:0]),
	.rst_n_in (rst_n_temp),
	.rst_n_out (rst_n));

  always @* begin
    instruction_reg <= instruction;    
  end
  initial begin
    instruction_reg <= 0;
    
  end
  
  


  
//   /* ram16_8 AUTO_TEMPLATE(
//  ); */
//ram16_8 U_USEQ_DATA_RAM (
//                        /*AUTOINST*/
//                         // Outputs
//                         .read_data           (read_data[15:0]),
//                         // Inputs
//                         .clock               (clock),
//                         .addr                (addr[ADDR_MSB:0]),
//                         .write_data          (write_data[15:0]),
//                         .cs                  (cs),
//                         .we                  (we[1:0]));
//
//
//

  


     /* clock_gen AUTO_TEMPLATE(
    ); */
  clock_gen U_CLOK_GEN (
                          /*AUTOINST*/
                        // Outputs
                        .clk            (clk));
						
 clock_gen PROG_CLOK_GEN (
                          /*AUTOINST*/
                        // Outputs
                        .clk            (prog_clk));
  
  

   /* reset_gen AUTO_TEMPLATE(
    .reset_n               (rst_n),
    ); */
  reset_gen U_RESET_GEN (
                          /*AUTOINST*/
                         // Outputs
                         .reset_n               (rst_n_temp));         // Templated
  
  


  task load_program_memory;
    reg [1024:0] filename;
    reg [7:0]    memory [2<<USEQ_ADDR_BITS:0]; // 8-bit memory
    integer      i;
    integer      dummy;
    begin

      filename = 0;
      dummy = $value$plusargs("program_memory=%s", filename);
      if(filename ==0) begin
        $display("WARNING! No content specified for program memory");
        end
      else begin
        $readmemh (filename, memory);
        for(i=0; i<(1<<USEQ_ADDR_BITS); i=i+2) begin
         mem0[i/2] = {memory[i+1],memory[i]};
        end
      end
    end
  endtask // load_program_memory

  event evt_finish;
  
  initial
    begin
      load_program_memory;
	  `ifdef NTL
      $dumpfile("tb_simdNTL.vcd");
      $dumpvars(0,tb_simd);
	  `else
	  $dumpfile("tb_simd.vcd");
      $dumpvars(0,tb_simd);
	  `endif
	  
    end
	always @(posedge prog_clk or negedge rst_n_temp) begin
	if(rst_n_temp == 1'b0) begin
		  started_prog <= 0;
		  count <=0;
		  instruction_in <= 0;
	end else
	if((load_program == 1'b1) && (started_prog ==1'b0)) begin
		count <= 0;
		started_prog <=1;
	end else if ((load_program ==1'b1) && (started_prog ==1'b1) && (count<30)) begin
		instruction_in <= mem0[count][15:0];
		count <= count + 1;
	end else begin
		load_program<=0;
		count <= 0;
		started_prog <=0;	
		instruction_in <= 0;		
	end
	end

  initial begin
	load_program = 1;
	sel_mode = 4'b0;
    #5000;
    $display("-E- Timeout reached !");
    -> evt_finish;
    #10;    
    $finish;
  end
 
`ifdef NTL
  reg halt;
  always @(negedge clk) begin
  if (instruction == 16'hE003) begin
	halt = 1;
	end else if(halt==1) begin
    $display("-I- Simulation halted by the CPU - <halt> instruction");
    -> evt_finish; 
    #10;    
    $finish;
	end else begin
		halt = 0;
	end
	end
    
  
   integer i;
   always @(evt_finish) begin
       $display("r00=0x%04h r01=0x%04h r02=0x%04h r03=0x%04h ",
                0,
                U_SIMD.U_USEQ.U_RF.regfile[223:208],
                U_SIMD.U_USEQ.U_RF.regfile[207:192],
                U_SIMD.U_USEQ.U_RF.regfile[191:176]);
       $display("r04=0x%04h r05=0x%04h r06=0x%04h r07=0x%04h ",
                U_SIMD.U_USEQ.U_RF.regfile[175:160],
                U_SIMD.U_USEQ.U_RF.regfile[159:144],
                U_SIMD.U_USEQ.U_RF.regfile[143:128],
                U_SIMD.U_USEQ.U_RF.regfile[127:112]);
       $display("r08=0x%04h r09=0x%04h r10=0x%04h r11=0x%04h ",
                U_SIMD.U_USEQ.U_RF.regfile[111:96],
                U_SIMD.U_USEQ.U_RF.regfile[95:80],
                U_SIMD.U_USEQ.U_RF.regfile[79:64],
                U_SIMD.U_USEQ.U_RF.regfile[63:48]);
       $display("r12=0x%04h r13=0x%04h r14=0x%04h pc=0x%04h ",
                U_SIMD.U_USEQ.U_RF.regfile[47:32],
                U_SIMD.U_USEQ.U_RF.regfile[31:16],
                U_SIMD.U_USEQ.U_RF.regfile[15:0],
                U_SIMD.U_USEQ.pc_r);
     end
  
`else
always @(posedge U_SIMD.U_USEQ.halt_r) begin
    $display("-I- Simulation halted by the CPU - <halt> instruction");
    -> evt_finish; 
    #10;    
    $finish;
  end
    
  
   integer i;
   always @(evt_finish) begin
       $display("r00=0x%04h r01=0x%04h r02=0x%04h r03=0x%04h ",
                0,
                U_SIMD.U_USEQ.U_RF.regfile[1],
                U_SIMD.U_USEQ.U_RF.regfile[2],
                U_SIMD.U_USEQ.U_RF.regfile[3]);
       $display("r04=0x%04h r05=0x%04h r06=0x%04h r07=0x%04h ",
                U_SIMD.U_USEQ.U_RF.regfile[4],
                U_SIMD.U_USEQ.U_RF.regfile[5],
                U_SIMD.U_USEQ.U_RF.regfile[6],
                U_SIMD.U_USEQ.U_RF.regfile[7]);
       $display("r08=0x%04h r09=0x%04h r10=0x%04h r11=0x%04h ",
                U_SIMD.U_USEQ.U_RF.regfile[8],
                U_SIMD.U_USEQ.U_RF.regfile[9],
                U_SIMD.U_USEQ.U_RF.regfile[10],
                U_SIMD.U_USEQ.U_RF.regfile[11]);
       $display("r12=0x%04h r13=0x%04h r14=0x%04h pc=0x%04h ",
                U_SIMD.U_USEQ.U_RF.regfile[12],
                U_SIMD.U_USEQ.U_RF.regfile[13],
                U_SIMD.U_USEQ.U_RF.regfile[14],
                U_SIMD.U_USEQ.pc_r);
     end
	`endif
  
endmodule // tb_pipeline
/*  
 Local Variables:
 verilog-library-directories:(
 "."
 "../../rtl/module/verilog"
 )
 End:
 */
